package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.io.File;

import java.util.Vector;
import java.util.Iterator;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Color;
import java.awt.Graphics;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JTextPane;
import javax.swing.text.StyledDocument;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

/**
 *  Used in the GUI interface for the MSS.
 */

public class ChooserPanel  extends JPanel implements ListSelectionListener {

    static Color lightBlue = new Color(0.8f,0.8f,1f);
    static Color lightGreen = new Color(0.7f,0.9f,1f);

    static StyledDocument emptyDoc = new DefaultStyledDocument();
    static {
        try {
            emptyDoc.insertString(0, "(No description)", null);
        }
        catch (Exception e) {
        }
    }

    static SimpleAttributeSet errorAttributes = new SimpleAttributeSet();
    static {
        StyleConstants.setForeground(errorAttributes, Color.red);
    }


    private JList listSelector;
    private JTextPane descriptionPane;

    static public void main(String[] args) {
        File propsDir = new File("data");
        File xmlDir = new File("style");
        
        JFrame jf = new JFrame();
        Container container = jf.getContentPane();

        jf.show();
    }


    public ChooserPanel(String title, ListItem[] items,
            ListSelectionListener selListener) {

        setBackground(lightBlue);
        Vector listItems = new Vector();
        Vector errorItems = new Vector();

        for (int i=0; i<items.length; ++i) {
            if (items[i].isValid()) {
                listItems.add(items[i]);
            }
            else {
                errorItems.add(items[i]);
            }
        }

        setLayout(new BorderLayout());

        // Title bar
        JLabel titleLabel = new JLabel(title);
        Font f = titleLabel.getFont();
        titleLabel.setFont(f.deriveFont(18f));
        add(titleLabel, BorderLayout.NORTH);

        // Scrolling List Selector
        listSelector = new JList(listItems);
        listSelector.setBackground(lightGreen);
        listSelector.addListSelectionListener(selListener);
        listSelector.addListSelectionListener(this);
        JScrollPane scrollPane = new JScrollPane();
        scrollPane.getViewport().setView(listSelector);
        add(scrollPane, BorderLayout.WEST);
        
        JComponent ep = createErrorPanel(errorItems);
        add(ep, BorderLayout.SOUTH);
    }


    public void valueChanged(ListSelectionEvent e) {
        ListItem li = (ListItem)listSelector.getSelectedValue();
        if (descriptionPane != null) {
            remove(descriptionPane);
            descriptionPane = null;
        }

        StyledDocument doc = li.getDescription();
        if (doc == null) {
            doc = emptyDoc;
        }

        descriptionPane = new JTextPane(doc);
        descriptionPane.setEditable(false);
        add(descriptionPane, BorderLayout.CENTER);
        invalidate();
    }


    private JComponent createErrorPanel(Vector items) {
        JTextPane jp = new JTextPane();
        jp.setEditable(false);
        jp.setBackground(new Color(0.9f,0.8f,0.8f));
        StringBuffer buf = new StringBuffer();
        Iterator iter = items.iterator();
        while (iter.hasNext()) {
            ListItem item = (ListItem)iter.next();
            buf.append(item.toString());
            buf.append(": ");
            buf.append(item.errorString());
            buf.append("\n\n");
        }
        DefaultStyledDocument errorDoc = new DefaultStyledDocument();
        try {
            errorDoc.insertString(0, buf.toString(), errorAttributes);
        }
        catch (javax.swing.text.BadLocationException e) {
        }
        jp.setDocument(errorDoc);
        
        return jp;
    }


    public ListItem getSelected() {
        return (ListItem)listSelector.getSelectedValue();
    }

}
