package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.util.Arrays;
import java.awt.RenderingHints;
import java.awt.image.RenderedImage;
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.renderable.RenderedImageFactory;

import javax.media.jai.ImageLayout;
import javax.media.jai.JAI;
import javax.media.jai.OperationDescriptor;
import javax.media.jai.OperationDescriptorImpl;
import javax.media.jai.OperationRegistry;
import javax.media.jai.ParameterListDescriptor;
import javax.media.jai.ParameterListDescriptorImpl;
import javax.media.jai.registry.RIFRegistry;

/**
 * An alternate implementation of functionality in the 'constant' operation.
 * But this implementation takes an {@link FImageLayout} object to define
 * the dimensions of the image to be created.  Also, the intensity is passed
 * in as a single value.  This allows for "pending" behavior
 * -- for the dimensions to be determined as a function of the pixel size.
 * Also, the intensity is passed in as a single value.
 */
public class Constant3OpDesc  extends OperationDescriptorImpl
                           implements RenderedImageFactory  {

    private static boolean isInitialized = false;
    public static final String OPERATION_NAME = "constant3";

    public static void init() {
        if (isInitialized)
            return;

        isInitialized = true;

        OperationRegistry opReg = 
                JAI.getDefaultInstance().getOperationRegistry();
        OperationDescriptor od = new Constant3OpDesc();
                
        opReg.registerDescriptor(od);
        RIFRegistry.register(opReg, od.getName(), OPERATION_NAME,
                             (RenderedImageFactory)od);
    }


    public Constant3OpDesc() {
        super(
                // resources
                new String[][] {
                     { "GlobalName",  OPERATION_NAME },
                     { "LocalName",  "constant3" },
                     { "Vendor",  "Attotron Biosensor Corporation" },
                     { "Description",  "Constant image accepts MaImageLayout" },
                     { "DocURL",  "" },
                     { "Version",  "0.9" },
                     { "arg0Desc",  "image layout" },
                     { "arg1Desc",  "intensity" },
                     { "arg2Desc",  "number of channels" }
                },

                // supported modes
                new String[] { "Rendered" },

                // source names
                null,   // use default naming convention "source0", ...

                // source classes
                new Class[][] { { RenderedImage.class } },

                new ParameterListDescriptorImpl(
                    // descriptor ??
                    new Object(),

                    // paramNames
                    new String[] { "fImageLayout", "intensity",
                            "numberOfChannels" },

                    // paramClasses
                    new Class[] { FImageLayout.class, Float.class,
                            NumberOfChannels.class },
                
                    // paramDefaults
                    new Object[] {
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT
                            },

                    // validParamValues
                    null
                )
        );

    }


    public int getNumSources() {
        return 0;
    }


    protected boolean validateSources(
            final String modeName,
            final ParameterBlock param,
            final StringBuffer message) {

        if (super.validateSources(modeName, param, message))  {
            return true;
        }
        
        return false;
    }


    protected boolean validateParameters(
            final String modeName,
            final ParameterBlock param,
            final StringBuffer message) {
        if (super.validateParameters(modeName, param, message))  {
            return true;
        }
        
        return false;
    }


    //  This is for RenderedImageFactory
    public RenderedImage create(final ParameterBlock params, 
                                final RenderingHints hints) {

        FImageLayout fImageLayout = (FImageLayout)params.getObjectParameter(0);
        float intensity = params.getFloatParameter(1);
        NumberOfChannels numberOfChannels =
                (NumberOfChannels)params.getObjectParameter(2);
        int numChan = numberOfChannels.numberOfChannels;

        RenderingHints rHints = FloatSampleModel.getRenderingHints(
                (int)fImageLayout.getMinX(),
                (int)fImageLayout.getMinY(), 
                (int)fImageLayout.getWidth(),
                (int)fImageLayout.getHeight(), 
                numChan);

        Float[] bandValues = new Float[numChan];
        Arrays.fill(bandValues, new Float(intensity));


        ParameterBlock pb = new ParameterBlock();
        pb.add(fImageLayout.getWidth());
        pb.add(fImageLayout.getHeight());
        pb.add(bandValues);
        RenderedImage rop = JAI.create("constant", pb, rHints);

        return rop;
    }

}
