package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import javax.swing.text.StyledDocument;
import javax.swing.text.DefaultStyledDocument;
import java.io.File;
import java.io.LineNumberReader;
import java.io.FileReader;
import java.util.regex.Pattern;


public class DataListItem extends File implements ListItem {

    static final String startPatternStr = "\\d+\t\\d+\t\\d+\t\\d+";
    static final String intensityPatternStr = "(0|1(?:\\.0*)|0?\\.\\d*)";
    static final Pattern commentLinePattern = Pattern.compile("\\A#");

    private boolean valid = true;
    private String errorStr = null;
    private int numChannels = 0;
    private String commentStr = null;
    private String lineTerminator = System.getProperty("line.separator");

    public DataListItem(File dataFile) {
         super(dataFile.getPath());
         try {
             checkFile();
         }
         catch (java.io.IOException e) {
             errorStr = e.getMessage();
             valid = false;
         }
    }


    public String toString() {
        return getName();
    }


    public String shortName() {
        String name = getName();
        int ind = name.lastIndexOf(".");
        return name.substring(0,ind);
    }


    public boolean isValid() {
        return valid;
    }


    public String errorString() {
        return errorStr;
    }


    public StyledDocument getDescription() {
        DefaultStyledDocument doc = new DefaultStyledDocument();
        String numChanStr = "(" + String.valueOf(numChannels) + " channels)\n";
        int offset = 0;
        try {
            String title = getName() + " " + numChanStr + "\n";
            doc.insertString(offset, title, null);
            offset += title.length();

/*
            doc.insertString(offset, numChanStr, null);
            offset += numChanStr.length();
*/

            if (commentStr != null) {
                doc.insertString(offset, commentStr, null);
                offset += commentStr.length();
            }
        }
        catch (javax.swing.text.BadLocationException e) {
        }
        return doc;
    }

    
    private void checkFile() throws java.io.IOException {
        Pattern linePattern = null;
        LineNumberReader rdr = null;
        try {
            rdr = new LineNumberReader(new FileReader(this));
        }
        catch (java.io.FileNotFoundException e) {
            throw new RuntimeException("Shouldn't get here");
        }

        StringBuffer commentBuf = new StringBuffer();
        String lineStr;

        while ((lineStr = rdr.readLine()) != null) {
            if (commentLinePattern.matcher(lineStr).find()) {
                commentBuf.append(lineStr.substring(1).trim());
                commentBuf.append(lineTerminator);
                continue;
            }

            if (numChannels == 0) {
                numChannels = lineStr.split("\t").length - 4;
                if (numChannels < 1 || numChannels > 2) {
                    errorStr = "Syntax error: unexpected number of columns";
                    valid = false;
                    return;
                }
                String patternStr =
                        startPatternStr + "\t" + intensityPatternStr +
                        ((numChannels == 2) ?  "\t" + intensityPatternStr : "");

                linePattern = Pattern.compile(patternStr);
            }
            if (!linePattern.matcher(lineStr).matches()) {
                errorStr = "Line #" + String.valueOf(rdr.getLineNumber()) + 
                        ": syntax error";
                valid = false;
                return;
            }
        }
        commentStr = commentBuf.toString();
    }

}
