package org.cybertory.MSS;

/*-------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.RenderingHints;
import java.awt.image.RenderedImage;
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.renderable.RenderedImageFactory;
import javax.media.jai.ImageLayout;
import javax.media.jai.JAI;
import javax.media.jai.OperationDescriptor;
import javax.media.jai.OperationDescriptorImpl;
import javax.media.jai.OperationRegistry;
import javax.media.jai.ParameterListDescriptor;
import javax.media.jai.ParameterListDescriptorImpl;
import javax.media.jai.registry.RIFRegistry;
import java.awt.image.BufferedImage;
import java.awt.Color;
//import java.awt.color.ColorSpace;
import java.awt.Polygon;
import javax.media.jai.InterpolationBilinear;
import javax.media.jai.RenderedOp;
import java.awt.Graphics2D;
import java.util.Random;


/**
 * Produce an image with random dirt specs.
 */
public class DirtOpDesc  extends OperationDescriptorImpl
                           implements RenderedImageFactory  {

    private static boolean isInitialized = false;
    public static final String OPERATION_NAME = "dirt";

    public static void init() {
        if (isInitialized)
            return;

        isInitialized = true;

        OperationRegistry opReg = 
                JAI.getDefaultInstance().getOperationRegistry();
        OperationDescriptor od = new DirtOpDesc();
                
        opReg.registerDescriptor(od);
        RIFRegistry.register(opReg, od.getName(), OPERATION_NAME,
                             (RenderedImageFactory)od);
    }


    public DirtOpDesc() {
        super(
                // resources
                new String[][] {
                     { "GlobalName",  OPERATION_NAME },
                     { "LocalName",  "Dirt" },
                     { "Vendor",  "Attotron Biosensor Corporation" },
                     { "Description", 
                             "Image with dirt specks, useful for background" },
                     { "DocURL",  "" },
                     { "Version",  "0.9" },
                     { "arg0Desc",  "image layout" },
                     { "arg1Desc",  "spacing (sq microns per dirt speck)" },
                     { "arg2Desc",
                             "average speck size (diameter in microns)" },
                     { "arg3Desc",
                             "standard deviation in speck size (in microns)" },
                     { "arg4Desc",  "average intensity" },
                     { "arg5Desc",  "standard deviation of intensity" },
                     { "arg6Desc",  "random number generator" }
                },

                // supported modes
                new String[] { "Rendered" },

                // source names
                null,   // use default naming convention "source0", ...

                // source classes
                new Class[][] { { RenderedImage.class } },

                new ParameterListDescriptorImpl(
                    // descriptor ??
                    new Object(),

                    // paramNames
                    new String[] { "fImageLayout", "spacing",
                                   "averageSize", "stddevSize",
                                   "averageIntensity", "stddevIntensity",
                                   "random" },

                    // paramClasses
                    new Class[] { FImageLayout.class, Float.class,
                                  Float.class, Float.class, 
                                  Float.class, Float.class,
                                  Random.class },
                
                    // paramDefaults
                    new Object[] {
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            null },

                    // validParamValues
                    null
                )
        );

    }


    public int getNumSources() {
        return 0;
    }


    protected boolean validateSources(
            final String modeName,
            final ParameterBlock param,
            final StringBuffer message) {

        if (super.validateSources(modeName, param, message))  {
            return true;
        }
        
        return false;
    }


/*
    protected boolean validateParameters(
            final String modeName,
            final ParameterBlock param,
            final StringBuffer message) {
        if (super.validateParameters(modeName, param, message))  {
            return true;
        }
        
        return false;
    }
*/


    //  This is for RenderedImageFactory
    public RenderedImage create(final ParameterBlock params, 
                                final RenderingHints hints) {

        int i=0;
        FImageLayout fImageLayout =
                (FImageLayout)params.getObjectParameter(i++);
        float spacing = params.getFloatParameter(i++);
        float averageSize = params.getFloatParameter(i++);
        float stddevSize = params.getFloatParameter(i++);
        float averageIntensity = params.getFloatParameter(i++);
        float stddevIntensity = params.getFloatParameter(i++);
        Random rand = (Random)params.getObjectParameter(i++);
        if (rand == null) {
            rand = new Random();
        }

        int width = (int)fImageLayout.getWidth();
        int height = (int)fImageLayout.getHeight();

        ImageLayout il = new ImageLayout((int)fImageLayout.getMinX(),
                (int)fImageLayout.getMinY(), width, height);
        FloatSampleModel sm = new FloatSampleModel(width, height, 1);
        il.setSampleModel(sm);

        RenderingHints rHints = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);

        int numSpecks = (int)(fImageLayout.getWidth() *
                fImageLayout.getHeight() / spacing);

        BufferedImage bi = dirtImage(width, height, numSpecks, averageSize,
                stddevSize, rand);

        ParameterBlock pb;

        pb = new ParameterBlock();
        pb.addSource(bi);
        pb.add(fImageLayout.getMinX());
        pb.add(fImageLayout.getMinY());
        pb.add(new InterpolationBilinear());
        RenderedOp rop = JAI.create("translate", pb);

        double offset =
                (double)(averageIntensity - stddevIntensity * 128f/50f);
        double pConst = (double)stddevIntensity/50.0;

        pb = new ParameterBlock();
        pb.addSource(rop);
        pb.add(new double[] {pConst});
        pb.add(new double[] {offset});
        pb.add(new InterpolationBilinear());
        rop = JAI.create("rescale", pb, rHints);

        return rop;
    }


    /*
     *  Create an 8-bit grayscale image of dirt specks.
     *  The intensity of specks has an average of 128 and a 
     *  standard deviation of 50.
     *  The size is specified in parameters.
     */
    private BufferedImage dirtImage(
            int width, 
            int height, 
            int numSpecks, 
            float averageSize,
            float stddevSize,
            Random rand) {

        BufferedImage bi = new BufferedImage(width, height,
                BufferedImage.TYPE_BYTE_GRAY);
        Graphics2D g = bi.createGraphics();

        PolygonFactory polyFact = new PolygonFactory(
                rand,
                0.5f * stddevSize,
                0.2f,  // radius stickiness
                11,
                0.5f);

        for (int i=0; i<numSpecks; ++i) {
            int x = rand.nextInt(width-1);
            int y = rand.nextInt(height-1);

            Polygon poly = polyFact.createPolygon(averageSize * 0.5f);
            poly.translate(x, y);
            float intensity = 128.0f + 50.0f * (float)rand.nextGaussian();

            intensity /= 256.0f;
            if (intensity < 0.0f) {
                intensity = 0.0f;
            }
            if (intensity > 1.0f) {
                intensity = 1.0f;
            }
            
            g.setColor(new Color(intensity, intensity, intensity));
            g.fill(poly);
        }
        
        return bi;
    }

}
