package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.media.jai.OperationDescriptorImpl;


/**
 *  A utility class for loading classes from a jar file.
 */
public class ExtensionsLoader {

    static void loadClasses(JarFile extensionsJar)  {
        Class cls;
        URLClassLoader ldr = null;

        try {
            URL url = new URL("jar:file:" + extensionsJar.getName() + "!/");
            ldr = new URLClassLoader(new URL[] {url});
        }
        catch (java.net.MalformedURLException e) {
            throw new RuntimeException("Shouldn't get here");
        }

        Enumeration enum = extensionsJar.entries();
        while (enum.hasMoreElements()) {
            JarEntry je = (JarEntry)enum.nextElement();
            String entryName = je.getName();
            if (entryName.endsWith(".class")) {
                int len = entryName.length();
                String className = entryName.substring(0, len-6);
                try {
                    cls = Class.forName(className, true, ldr);
                }
                catch (ClassNotFoundException e) {
                    throw new RuntimeException("Shouldn't get here");
                }
                processClass(cls);
            }
        }
    }


    static private Object createInstance(Class cls) {
        try {
            Constructor constr = cls.getConstructor(new Class[] {});
            return constr.newInstance(new Object[] {});
        }
        catch (InvocationTargetException e) {
            System.out.println(e.getCause());
            throw new RuntimeException("ABORT");
        }
        catch (Throwable e) {
            System.out.println(e);
            System.out.println("Class " + cls.getName());
            throw new RuntimeException("ABORT");
        }
    }


    static private void processClass(Class cls) {
        if (XMLHandler.class.isAssignableFrom(cls)) {
            createInstance(cls);
        }

        else if (X_OperationJAI.class.isAssignableFrom(cls) ||
                OperationDescriptorImpl.class.isAssignableFrom(cls)) {

            try {
                X_OperationJAI op =
                        (X_OperationJAI.class.isAssignableFrom(cls)) ?
                                (X_OperationJAI)createInstance(cls) :
                        new X_OperationJAI(
                                (OperationDescriptorImpl)createInstance(cls));
                X_PendingRenderedOp.registerOperation(op);
            }

            catch (XMLUnhandledTypeException e) {
                System.out.print(cls.getName() + ":");
                System.out.println(e);
                throw new RuntimeException("ABORT");
            }
        }
    }

}
