package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.lang.reflect.Constructor;
import java.util.Hashtable;
import java.util.Map;

/**
 *  An implementation of {@link ExternalDataFactory} that gets its 
 *  data from the java command line.
 */
public class ExternalDataFactoryImpl extends Hashtable
        implements ExternalDataFactory {

    public ExternalDataFactoryImpl(Map m) {
        super(m);
    }


    public ExternalDataFactoryImpl() {
    }


    /**
     *  Return a data object by key.
     *
     *  @param key   string identifying the external data object of interest
     *  @param oType the expected data type for the object
     *  @return      the object for the given key
     */
    public Object getExternalData(String key, Class oType)
            throws ExternalDataException {

        String sValue = (String)get(key);
        if (sValue == null || oType.isAssignableFrom(String.class)) {
            return sValue;
        }

        try {
            Constructor constructor =
                   oType.getConstructor(new Class[] { String.class });
            return constructor.newInstance(new Object[] { sValue });
        }
        catch (NoSuchMethodException e) {
            throw new ExternalDataException("Cannot parse type " + 
                    oType.getName() + " from string", e);
            
        }
        catch (Exception e) {
            throw new ExternalDataException("Data format error", e);
        }

    }

}
