package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;


public class ExternalDataPanel extends JPanel implements ActionListener {

    final private Hashtable dataMap;
    private LabeledTextField[] tFields;

    public ExternalDataPanel(String[] externalKeys, Hashtable pDataMap) {
        this.dataMap = (pDataMap == null) ? new Hashtable() : pDataMap;

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        tFields = new LabeledTextField[externalKeys.length];
        for (int i=0; i<externalKeys.length; ++i) {
            String key = externalKeys[i];
            String value = null;
            if (dataMap != null) {
                value = (String)dataMap.get(key);
            }

            LabeledTextField tField = new LabeledTextField(key, value);
            tFields[i] = tField;
            add(tField);
        }
    }


    public void actionPerformed(ActionEvent event) {
        for (int i=0; i<tFields.length; ++i) {
            String key = tFields[i].label;
            String value = tFields[i].getText();

            if (value == null || value.length() == 0) {
                if (dataMap.containsKey(key)) {
                    dataMap.remove(key);
                }
            }

            else {
                dataMap.put(key, value);
            }
        }
    }


    public class LabeledTextField extends JPanel {
        public final String label;
        private JTextField tField;

        public LabeledTextField(String label) {
            this(label, null);
        }


        public LabeledTextField(String label, String initValue) {
            setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
            this.label = label;
            add(new JLabel(label + " "));
            tField = new JTextField(initValue, 40);
            tField.setPreferredSize(tField.getMinimumSize());
            add(tField);
        }

        public String getText() {
            return tField.getText();
        }
    }

}
