package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.image.RenderedImage;

import javax.media.jai.ImageLayout;

/**  Subset of ImageLayout, but with float values instead of int values. */

public class FImageLayout implements SelfScaler {

    private float minX;
    private float minY;
    private float width;
    private float height;

    private PendingRenderedOp rop = null;


    public FImageLayout(PendingRenderedOp rop) {
        this.rop = rop;
    }


    public FImageLayout(RenderedImage ri) {
        this((float)ri.getMinX(), (float)ri.getMinY(),
                (float)ri.getWidth(), (float)ri.getHeight());
    }


    public FImageLayout(ImageLayout il) {
        this.minX = (float)il.getMinX(null);
        this.minY = (float)il.getMinY(null);
        this.width = (float)il.getWidth(null);
        this.height = (float)il.getHeight(null);
    }


    public FImageLayout(float minX, float minY, float width, float height) {
        this.minX = minX;
        this.minY = minY;
        this.width = width;
        this.height = height;
    }


    public Object getScaled(float pixelSize) {
        if (rop != null) {
            PendingRenderedOp scaledRop = (PendingRenderedOp)
                    rop.getScaled(pixelSize);
            return scaledRop.getLayout();
        }
        return new FImageLayout(minX/pixelSize, minY/pixelSize,
                width/pixelSize, height/pixelSize);
    }


    public ImageLayout getImageLayout() {
        return new ImageLayout((int)minX, (int)minY, (int)width, (int)height);
    }


    public float getMinX() {
        return minX;
    }


    public float getMinY() {
        return minY;
    }


    public float getWidth() {
        return width;
    }


    public float getHeight() {
        return height;
    }

}
