package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
/**
 *  A standard implmentation of FeaturesIntensities that uses a constant
 *  multiple dimension array of pixel values.
 */
public class FeaturesIntensitiesImpl implements FeaturesIntensities {

    private final float[][][][] intensities;


    static private float[][][] handleZone(RawIntensitiesRecord[] rawRecs,
            int startInd, int numRecs) {
        int maxRow = -1;
        int maxCol = -1;
        for (int i=startInd; i<startInd+numRecs; ++i) {
            RawIntensitiesRecord rec = rawRecs[i];
            if (rec.row > maxRow) {
                maxRow = rec.row;
            }
            if (rec.col > maxCol) {
                maxCol = rec.col;
            }
        }
        float[][][] zIntensities = new float[maxRow+1][maxCol+1][];
        for (int i=startInd; i<startInd+numRecs; ++i) {
            RawIntensitiesRecord rec = rawRecs[i];
            zIntensities[rec.row][rec.col] = rec.intensities;
        }
        return zIntensities;
    }


    public FeaturesIntensitiesImpl(RawIntensitiesData rawData) {
        RawIntensitiesRecord[] rawRecs = rawData.getData();
        fillInZoneIndex(rawRecs);
        int numZones = rawRecs[rawRecs.length-1].zoneIndex + 1;

        intensities = new float[numZones][][][];
        int currZone = -1;
        int startZone = 0;
        for (int i=0; i<rawRecs.length; ++i) {

            if (rawRecs[i].zoneIndex == -1) {
            }

            if (rawRecs[i].zoneIndex != currZone) {
                if (currZone != -1) {
                    intensities[currZone] =
                            handleZone(rawRecs, startZone, i-startZone);
                }
                startZone = i;
                currZone = rawRecs[i].zoneIndex;
            }
        }
        intensities[currZone] =
                handleZone(rawRecs, startZone, rawRecs.length-startZone);
    }


    static private void fillInZoneIndex(RawIntensitiesRecord[] rawRecs) {
        if (rawRecs[0].zoneIndex >= 0) {
            return;
        }

        int zoneIndex = -1;
        int metaRow = -1;
        int metaCol = -1;
        for (int i=0; i<rawRecs.length; ++i) {
            if (rawRecs[i].metarow != metaRow ||
                    rawRecs[i].metacol != metaCol) {
                ++zoneIndex;
                metaRow = rawRecs[i].metarow;
                metaCol = rawRecs[i].metacol;
            }
            rawRecs[i].zoneIndex = zoneIndex;
        }
    }


    public FeaturesIntensitiesImpl(float[][][][] intensities) {
        this.intensities = intensities;
    }


    public int getNumZones() {
        return intensities.length;
    }


    public float[][][] intensitiesForZone(int zoneIndex) {
        return intensities[zoneIndex];
    }
}
