package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.RenderingHints;
import java.awt.image.RenderedImage;
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.renderable.RenderedImageFactory;

import javax.media.jai.JAI;
import javax.media.jai.OperationDescriptor;
import javax.media.jai.OperationDescriptorImpl;
import javax.media.jai.OperationRegistry;
import javax.media.jai.ParameterListDescriptorImpl;
import javax.media.jai.operator.MosaicDescriptor;
import javax.media.jai.registry.RIFRegistry;

/**
 *  An operation for Gaussian blur.
 */
public class FirstNonzeroOpDesc  extends OperationDescriptorImpl
                           implements RenderedImageFactory  {

    private static boolean isInitialized = false;
    public static final String OPERATION_NAME = "FirstNonzero";

    public static void init() {
        if (isInitialized)
            return;

        isInitialized = true;

        OperationRegistry opReg = 
                JAI.getDefaultInstance().getOperationRegistry();
        OperationDescriptor od = new FirstNonzeroOpDesc();
                
        opReg.registerDescriptor(od);
        RIFRegistry.register(opReg, od.getName(), OPERATION_NAME,
                             (RenderedImageFactory)od);
    }


    public FirstNonzeroOpDesc() {
        super(
                // resources
                new String[][] {
                     { "GlobalName",  OPERATION_NAME },
                     { "LocalName",  "FirstNonzero" },
                     { "Vendor",  "Attotron Biosensor Corporation" },
                     { "Description",
                "Compositing function taking first nonzero pixel value" },
                     { "DocURL",  "" },
                     { "Version",  "0.9" },
                },

                // supported modes
                new String[] { "Rendered" },

                // source names
                null,   // use default naming convention "source0", ...

                // source classes
                new Class[][] { { RenderedImage.class } },

                new ParameterListDescriptorImpl(
                    // descriptor ??
                    new Object(),

                    // paramNames
                    new String[] { },

                    // paramClasses
                    new Class[] { },
                
                    // paramDefaults
                    null,

                    // validParamValues
                    null
                )
        );

    }


    //  This is for RenderedImageFactory
    public RenderedImage create(final ParameterBlock params, 
                                final RenderingHints hints) {

        ParameterBlock pb = new ParameterBlock();
        pb.add(MosaicDescriptor.MOSAIC_TYPE_OVERLAY);
        pb.add(null);  // sourceAlpha
        pb.add(null);  // sourceROI
        pb.add( new double[][] { { 1e-50 } } );

        pb.setSources(params.getSources());
        RenderedImage nonZeroOp = (hints == null) ?
                JAI.create("mosaic", pb) : JAI.create("mosaic", pb, hints);

        return nonZeroOp;
    }

}
