package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

/**
 *  This function assumes the target image has an origin of (0,0) and
 *  a lower left corner of (1,1).
 *
 *
 */
public class GradientFunction  extends SimpleFunction {

    private double gradientAngle;
    private double centerIntensity;
    private double rateOfIntensityChange;
    private AffineTransform rotator;

    public GradientFunction(
            double gradientAngle,
            double centerIntensity,
            double rateOfIntensityChange) {

        this.gradientAngle = gradientAngle;
        this.centerIntensity = centerIntensity;
        this.rateOfIntensityChange = rateOfIntensityChange;

        rotator =
                AffineTransform.getRotateInstance(gradientAngle, 0.5, 0.5);
    }


    public double calcValue(double x, double y) {
        Point2D orig = new Point2D.Double(x,y);
        Point2D rotPoint = new Point2D.Double(x,y);
        rotator.transform(orig, rotPoint);

        double normX = rotPoint.getX() - 0.5;
        return centerIntensity + normX * rateOfIntensityChange;
    }


    public Object getScaled(float pixelSize) {
        return new GradientFunction(
            gradientAngle,
            centerIntensity / pixelSize,
            rateOfIntensityChange / pixelSize);
    }

}
