package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.Dimension;
import java.awt.Point;
import java.util.Vector;

/**
 *  For microarrays where all zones are the same size, located on a regular
 *  grid (evenly spaced metarows and metacols).
 *  Also, the spacing between rows and columns (within zones) is consistent
 *  amongst all the zones.
 */
public class GridFeaturesIntensities implements FeaturesIntensities {

    public final int numMetarows;
    public final int numMetacols;
    public final int numRows;
    public final int numCols;
    public final Point[] zoneMetaCoordinates;

    private final float[][][][] intensities;


    public GridFeaturesIntensities(float[][][][] intensities, 
            int numMetarows, int numMetacols, int numRows, int numCols,
            Point[] zoneMetaCoords) {

        this.intensities = intensities;
        this.zoneMetaCoordinates = zoneMetaCoords;
        this.numMetarows = numMetarows;
        this.numMetacols = numMetacols;
        this.numRows = numRows;
        this.numCols = numCols;
    }


    public GridFeaturesIntensities(float[][][][][] gridIntensities,
            int numRows, int numCols) {
        numMetarows = gridIntensities.length;
        numMetacols = gridIntensities[0].length;
        this.numRows = numRows;
        this.numCols = numCols;

        Vector vZones = new Vector();
        Vector vZonesCoords = new Vector();
        for (int mRow=0; mRow<numMetarows; ++mRow) {
            for (int mCol=0; mCol<numMetacols; ++mCol) {
               if (gridIntensities[mRow][mCol] != null) {
                   vZonesCoords.add(new Point(mCol,mRow));
                   vZones.add(gridIntensities[mRow][mCol]);
               } 
            }
        }
        intensities = (float[][][][])
                vZones.toArray(new float[vZones.size()][][][]);
        zoneMetaCoordinates = (Point[])
                vZonesCoords.toArray(new Point[vZonesCoords.size()]);
    }


    public int getNumZones() {
        return intensities.length;
    }


    public float[][][] intensitiesForZone(int zoneIndex) {
        return intensities[zoneIndex];
    }


    public Dimension getMetaDimension() {
        return new Dimension(numMetacols, numMetarows);
    }


    public Dimension getZoneDimension() {
        return new Dimension(numCols, numRows);
    }


    public Point getZoneMetaCoordinate(int zoneIndex) {
        return zoneMetaCoordinates[zoneIndex];
    }


    public Point[] getZoneMetaCoordinates() {
        return zoneMetaCoordinates;
    }

}
