package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.RenderingHints;
import java.awt.image.RenderedImage;
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.renderable.RenderedImageFactory;

import javax.media.jai.ImageLayout;
import javax.media.jai.JAI;
import javax.media.jai.OperationDescriptor;
import javax.media.jai.OperationDescriptorImpl;
import javax.media.jai.OperationRegistry;
import javax.media.jai.ParameterListDescriptor;
import javax.media.jai.ParameterListDescriptorImpl;
import javax.media.jai.registry.RIFRegistry;
import javax.media.jai.ImageFunction;


/**
 * An alternate implementation of functionality in the 'constant' operation.
 * But this implementation takes an {@link FImageLayout} object to define
 * the dimensions of the image to be created.  This allows for "pending"
 * behavior -- for the dimensions to be determined as a function of the
 * pixel size.
 */
public class ImageFunction2OpDesc  extends OperationDescriptorImpl
                           implements RenderedImageFactory  {

    private static boolean isInitialized = false;
    public static final String OPERATION_NAME = "imagefunction2";

    public static void init() {
        if (isInitialized)
            return;

        isInitialized = true;

        OperationRegistry opReg = 
                JAI.getDefaultInstance().getOperationRegistry();
        OperationDescriptor od = new ImageFunction2OpDesc();
                
        opReg.registerDescriptor(od);
        RIFRegistry.register(opReg, od.getName(), OPERATION_NAME,
                             (RenderedImageFactory)od);
    }


    public ImageFunction2OpDesc() {
        super(
                // resources
                new String[][] {
                     { "GlobalName",  OPERATION_NAME },
                     { "LocalName",  "ImageFunction2" },
                     { "Vendor",  "Attotron Biosensor Corporation" },
                     { "Description",
                            "ImageFunction image accepts MaImageLayout" },
                     { "DocURL",  "" },
                     { "Version",  "0.9" },
                     { "arg0Desc",  "band values" },
                     { "arg1Desc",  "image function" },
                },

                // supported modes
                new String[] { "Rendered" },

                // source names
                null,   // use default naming convention "source0", ...

                // source classes
                new Class[][] { { RenderedImage.class } },

                new ParameterListDescriptorImpl(
                    // descriptor ??
                    new Object(),

                    // paramNames
                    new String[] { "function", "fImageLayout" },

                    // paramClasses
                    new Class[] { ImageFunction.class, FImageLayout.class },
                
                    // paramDefaults
                    new Object[] {
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT,
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT },

                    // validParamValues
                    null
                )
        );

    }


    public int getNumSources() {
        return 0;
    }


    protected boolean validateSources(
            final String modeName,
            final ParameterBlock param,
            final StringBuffer message) {

        if (super.validateSources(modeName, param, message))  {
            return true;
        }
        
        return false;
    }


    protected boolean validateParameters(
            final String modeName,
            final ParameterBlock param,
            final StringBuffer message) {
        if (super.validateParameters(modeName, param, message))  {
            return true;
        }
        
        return false;
    }


    //  This is for RenderedImageFactory
    public RenderedImage create(final ParameterBlock params, 
                                final RenderingHints hints) {

        ImageFunction iFunction = (ImageFunction)params.getObjectParameter(0);
        FImageLayout fImageLayout = (FImageLayout)params.getObjectParameter(1);

        ImageLayout il = new ImageLayout(
                (int)fImageLayout.getMinX(),
                (int)fImageLayout.getMinY(), 
                (int)fImageLayout.getWidth(),
                (int)fImageLayout.getHeight());

        il.setSampleModel(new FloatSampleModel(
                (int)fImageLayout.getWidth(),
                (int)fImageLayout.getHeight(),
                1));

        RenderingHints rHints = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);

        float xScale = 1f/(fImageLayout.getWidth() - fImageLayout.getMinX());
        float yScale = 1f/(fImageLayout.getHeight() - fImageLayout.getMinY());

        ParameterBlock pb = new ParameterBlock();
        pb.add(iFunction);
        pb.add((int)fImageLayout.getWidth());
        pb.add((int)fImageLayout.getHeight());
        pb.add(xScale);
        pb.add(yScale);
        pb.add(fImageLayout.getMinX());
        pb.add(fImageLayout.getMinY());

        RenderedImage rop = JAI.create("imagefunction", pb, rHints);

        return rop;
    }

}
