package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.RenderingHints;
import java.awt.image.ComponentSampleModel;
import java.awt.image.DataBuffer;
import java.awt.image.renderable.ParameterBlock;

import javax.media.jai.ImageLayout;
import javax.media.jai.JAI;
import javax.media.jai.RenderedOp;

/**
 *  Save type for saving an image as a JPEG file.
 */
public class Jpeg extends ScanSaveType {

    static public final String defaultFilenameExtension = "jpeg";
    
    private final float resizeFactor;


    public Jpeg(CommonScanSaveParams commonParams, float resizeFactor) {
        super(commonParams);
        this.resizeFactor = resizeFactor;
    }


    /**
     * Create the color mapping matrix for the BandCombine operator to
     * render a color (RGB for jpeg) representation of the image.
     *
     * @return a 2-dim matrix describing color channel mapping
     */
    private double[][] createColorMapMatrix(int numBands) {
        
        switch (numBands) {
            case 1:  return new double[][] {
                { 255.0, 0.0 },
                { 255.0, 0.0 },
                { 255.0, 0.0 } };

            case 2:  return new double[][] {
                { 0.0, 255.0, 0.0 },
                { 255.0, 0.0, 0.0 },
                { 0.0, 0.0, 0.0 } };

            case 3:  return new double[][] {
                { 0.0, 255.0, 0.0, 0.0 },
                { 255.0, 0.0, 0.0, 0.0 },
                { 0.0, 0.0, 255.0, 0.0 } };
        }
        return null;  // appease compiler
    }


    public void createFile(RenderedOp image, String fileStem) {

        int numBands = image.getSampleModel().getNumBands();
        if (numBands > 3) {
            System.out.println("Can't create jpeg images for experiments with"
                    + "more than 3 channels.");
            return;
        }

        //  Indicate the sample model and size of the output image
        ImageLayout il = new ImageLayout();
        int width = image.getWidth();
        int height = image.getHeight();
        ComponentSampleModel sm = new ComponentSampleModel(
                DataBuffer.TYPE_BYTE, width, height, 3, 3 * width,
                new int[] {0,1,2} );
        il.setSampleModel(sm);
        RenderingHints rh = new RenderingHints(JAI.KEY_IMAGE_LAYOUT, il);

        double[][] matrix = createColorMapMatrix(numBands);
        ParameterBlock pb = new ParameterBlock();
        pb.addSource(image);
        pb.add(matrix);
        RenderedOp rgbOp = JAI.create("BandCombine", pb, rh);

        RenderedOp finalImage = rgbOp;
        if (resizeFactor != 1f) {
            pb = new ParameterBlock();
            pb.addSource(rgbOp);
            pb.add(resizeFactor);
            pb.add(resizeFactor);
            finalImage = JAI.create("Scale", pb);
        }

        pb = new ParameterBlock();
        pb.addSource(finalImage);
        pb.add(imagePath(fileStem));
        pb.add("jpeg");
        JAI.create("FileStore", pb);
    }

}
