package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Properties;
import java.util.jar.JarFile;

import org.jdom.Document;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/** 
 *  Microarray scan command line handler.
 */
public class MSSCommandLine {

    static public void processParams(String[] args) throws Exception {

        if (args.length == 1 && args[0].equals("?") ) {
            printHelp();
            System.exit(0);;
        }

        HashMap pMap = new HashMap();
        HashMap dMap = new HashMap();

        int i=0;
        while (i<args.length && args[i].startsWith("-")) {
            String key = args[i++].substring(1);

            if (key.equals("help")) {
                printHelp();
                System.exit(0);;
            }

            if (key.equals("schema")) {
                //schema = true;
                continue;
            }

            String value = null;
            if (i<args.length && !args[i].startsWith("-")) {
                value = args[i++];
            }

            if (key.startsWith("D:")) {
                dMap.put(key.substring(2), value);
            }
            else {
                pMap.put(key, value);
            }
        }
        String xmlFName = (i==args.length) ? null: args[args.length-1];

        if (xmlFName == null) {
        }

        String extensions = (String)pMap.get("extensions");
        if (extensions != null) {
            File extensionsFile = new File(extensions);
            try {
                ExtensionsLoader.loadClasses( new JarFile(extensionsFile) );
            }
            catch (java.io.IOException e) {
                throw new RuntimeException(extensions+" isn't valid jar file");
            }
        }


        if (xmlFName == null) {
            Document doc = XMLHandler.schemaDocument();
            XMLOutputter xo = new XMLOutputter();
            xo.setFormat( Format.getPrettyFormat() );
            xo.output(doc, System.out);
        }

        else {
            ExternalDataFactoryImpl edFac = argsFactory(pMap, dMap);
            handleImages(xmlFName, new ExternalDataFactory[] { edFac });
        }
    }


    static private ExternalDataFactoryImpl argsFactory(HashMap pMap,
            HashMap dMap)  {

        ExternalDataFactoryImpl edFac = null;
        String extDataFName = (String)pMap.get("D");
        if (extDataFName != null) {
            try {
                File propertiesFile = new File(extDataFName);
                Properties props = new Properties();
                props.load(new FileInputStream(propertiesFile));
                edFac = new ExternalDataFactoryImpl(props);
            }
            catch (Exception e) {
                throw new RuntimeException(
                        "Error in processing properties file " +
                        extDataFName);
            }
        }

        if (edFac == null) {
            edFac = new ExternalDataFactoryImpl(dMap);
        }
        else {
            edFac.putAll(dMap);
        }

        return edFac;
    }


    static private void handleImages(String xmlFName,
            ExternalDataFactory[] edFactories) {

        InputStream xmlFile = System.in;

        Document xmlDoc = null;
        try {
            if (xmlFName != null) {
                xmlFile = new FileInputStream(xmlFName);
            }
            xmlDoc = new SAXBuilder().build(xmlFile);
        }
        catch (Exception e) {
            System.out.println(e);
        }

        try {
            Scan[] scans = XMLHandler.parseXMLDocument(xmlDoc, edFactories);
            for (int i=0; i<scans.length; ++i) {
                scans[i].saveImages();
            }
        }
        catch (XMLParseException e) {
            System.out.println(e);
            e.printStackTrace();
            Throwable t = e.getCause();
            if (t != null) {
            	t.printStackTrace();
            }
            throw new RuntimeException();
        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
                throw new RuntimeException();
        }
    }


    static private void printHelp() {

        System.out.println(
                                                                          "\n" +
                                                                          "\n" +
            "java -jar MSS.jar ?"                                       + "\n" +
            "       OR"                                                 + "\n" +
            "java -jar MSS.jar -help"                                   + "\n" +
            "   to see this usage description"                          + "\n" +
                                                                          "\n" +
            "Usage for outputting schema (sent to stdout):"             + "\n" +
            "  java -jar MSS.jar [-extensions <extensions_jar>]"        + "\n" +
            "                   [-schema]"                              + "\n" +
                                                                          "\n" +
            "    where"                                                 + "\n" +
            "      <extensions_jar> contains extension classes"         + "\n" +
                                                                          "\n" +
            "          OR" +                                              "\n" +
                                                                          "\n" +
            "Usage for generating images:"                              + "\n" +
            " java -jar MSS.jar [-extensions <extensions_jar>]"         + "\n" +
            "   [-D:<key>=<value>]... [-D <properties_file>] <xml_file>"+ "\n" +
                                                                          "\n" +
            "    where"                                                 + "\n" +
            "      <key>,<value> are key value pairs for external data" + "\n" +
            "      <properties_file> contains key/value pairs for external data"
                                                                        + "\n" +
            "      <xml_file> is an XML file describing the image scans"+ "\n"
        );

    }

}
