package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.lang.NullPointerException;

import java.awt.Color;
import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import java.util.Properties;
import java.util.jar.JarFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.jdom.output.XMLOutputter;
import org.jdom.output.Format;
import org.jdom.Document;

import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.border.MatteBorder;
import javax.swing.BorderFactory;


/** 
 *  Microarray scan simulation GUI interface
 */
public class MSSGUI extends JFrame
        implements ListSelectionListener, ActionListener {

    static File currDir;
    static {
        try {
            currDir = new File(System.getProperty("user.dir"));
        }
        catch (Exception e) {
            throw new RuntimeException("Shouldn't get here");
        }
    }

    static private Border pBorder = new MatteBorder(15, 15, 15, 15,
                new Color(0.7f,0.7f,0.9f));

    private ChooserPanel propsPanel;
    private ChooserPanel dataPanel;
    private JButton acceptButton;
    private JTextField outputFileStemField;

    private JButton extensionsButton;
    private JButton propertiesButton;
    private JButton schemaButton;
    private Container container;
    //private Document xmlDoc = null;



    public MSSGUI() {
        super("Microarray Scan Simulator");

        loadExtensions();
        
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setSize(800,600);
        container = getContentPane();
        initialDisplay();
    }


    private void initialDisplay() {
        container.removeAll();
        container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));

        //--------------------------
        // Properties File Chooser
        //--------------------------
        File styleDir = new File(currDir, "style");
        File[] propFiles = styleDir.listFiles(new FileExtensionFilter(
                "properties"));
        PropertiesListItem[] propsItems =
                new PropertiesListItem[propFiles.length];
        for (int i=0; i<propsItems.length; ++i) {
            propsItems[i] = new PropertiesListItem(propFiles[i], styleDir);
        }
        propsPanel = new ChooserPanel("Choose a style", propsItems,
                this);
        propsPanel.setBorder(pBorder);
        container.add(propsPanel);


        //--------------------------
        // Data Files Chooser
        //--------------------------
        File dataDir = new File(currDir, "data");
        File[] dataFiles = dataDir.listFiles(new FileExtensionFilter("txt"));
        DataListItem[] dataItems = new DataListItem[dataFiles.length];
        for (int i=0; i<dataItems.length; ++i) {
            dataItems[i] = new DataListItem(dataFiles[i]);
        }
        dataPanel = new ChooserPanel("Choose an intensities file", dataItems,
                this);
        dataPanel.setBorder(pBorder);
        container.add(dataPanel);


        //----------------------------------
        // Text field for entry of outputFileStem
        //----------------------------------
        JPanel fieldPanel = new JPanel();
        fieldPanel.setLayout(new FlowLayout());
        outputFileStemField = new JTextField(25);
        fieldPanel.add(new JLabel("Enter stem for names of output files: "));
        fieldPanel.add(outputFileStemField);
        container.add(fieldPanel);


        //--------------------------
        // CONTINUE button
        //--------------------------
        JPanel buttonsPanel = new JPanel();
        buttonsPanel.setLayout(new FlowLayout());

        acceptButton = new JButton("CONTINUE");
        acceptButton.addActionListener(this);
        acceptButton.setEnabled(false);
        acceptButton.setActionCommand("CONTINUE");
        buttonsPanel.add(acceptButton);

        container.add(buttonsPanel);

        show();
    }


    /**
     *  Enable the Accept button  IFF
     *  both a properties file and a data file are selected.
     */
    public void valueChanged(ListSelectionEvent e) {
        acceptButton.setEnabled(propsPanel.getSelected() != null &&
                               dataPanel.getSelected() != null);
    }


    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();
        if (action.equals("CONTINUE")) {

            handleSelection();
        }

        else if (action.equals("genImages")) {
            PropertiesListItem pItem =
                    (PropertiesListItem)propsPanel.getSelected();
            generateImages(pItem);
        }

    }


    private void handleSelection() {
        PropertiesListItem pItem =
                (PropertiesListItem)propsPanel.getSelected();
        DataListItem dItem =
                (DataListItem)dataPanel.getSelected();
        pItem.put("intensitiesFile", dItem.getPath());

        String outputFileStem = outputFileStemField.getText();
        if (outputFileStem == null || outputFileStem.equals("")) {
            outputFileStem = (String)pItem.get("outputFileStem");
        }

        if (outputFileStem == null || outputFileStem.equals("")) {
            outputFileStem = pItem.toString() + "_" + dItem.shortName();
        }
        pItem.put("outputFileStem", outputFileStem);

        String[] keys = pItem.externalKeys();
        boolean missingData = false;
        for (int i=0; i<keys.length; ++i) {
            if (!pItem.containsKey(keys[i])) {
                missingData = true;
                break;
            }
        }

        if (missingData) {
            paramsPanel(pItem);
        }
        else {
            generateImages(pItem);
        }
    }


    private void loadExtensions() {
        File extFile = null;
        try {
            extFile = new File(currDir, "extensions.jar");
        }
        catch (NullPointerException e) {
            throw new RuntimeException("Shouldn't get here");
        }
       
        if (extFile.exists()) {
            try {
                ExtensionsLoader.loadClasses( new JarFile(extFile) );
            }
            catch (java.io.IOException e) {
                JOptionPane.showMessageDialog(this,
                        "Error in reading 'extensions.jar': " +
                        e.getMessage() );
            }
        }
    }


    private void waitForImagesPanel() {
        container.removeAll();
        //container.setLayout(new FlowLayout());

        JLabel label = new JLabel("Wait: images being generated");

        Font f = label.getFont();
        label.setFont(f.deriveFont(28f));

        container.add(label);

        show();
        paint(getGraphics());
    }


    private void paramsPanel(PropertiesListItem pItem) {
        container.removeAll();
        //container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
        container.setLayout(new BorderLayout());
    
        ExternalDataPanel edp =
                new ExternalDataPanel(pItem.externalKeys(), pItem);
        edp.setBorder(pBorder);
        container.add(edp, BorderLayout.NORTH);

        JButton genImages = new JButton("Generate Images");
        JPanel jp = new JPanel();
        jp.setLayout(new FlowLayout());

        genImages.setActionCommand("genImages");
        genImages.addActionListener(this);
        genImages.addActionListener(edp);
        jp.add(genImages);
        container.add(jp, BorderLayout.SOUTH);

        show();
        paint(getGraphics());
    }


    private void generateImages(PropertiesListItem pItem) {
        boolean success = true;
        waitForImagesPanel();
        Document xmlDoc = null;
        try {
            xmlDoc = pItem.getXMLDoc();
            Scan[] scans = XMLHandler.parseXMLDocument(xmlDoc,
                    new ExternalDataFactory[] { pItem } );
            for (int i=0; i<scans.length; ++i) {
                scans[i].saveImages();
            }
        }
        catch (Exception e) {
            success = false;
            JOptionPane.showMessageDialog(this, "Error: " + e.getMessage());
            System.out.println(e);
            e.printStackTrace();
            Throwable t = e.getCause();
            if (t != null) {
                t.printStackTrace();
            }
        }
        finally {
            if (success) {
                String outputDescr =
                        "Your images are ready in the 'images' directory " +
                        "with file stem '" +
                        (String)pItem.get("outputFileStem") +
                        "'.";
                JOptionPane.showMessageDialog(this, outputDescr);
            }
            initialDisplay();
        }
    }

}
