package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.Point;
import java.awt.geom.Point2D;

/**
 * This class encapsulates the layout of a microarray with 2 zones overlapping to
 * produce the effect of tight packing, such as in Agilent arrays.
 * The second zone is offset from the first by half the feature cell width and
 * height.
 */
public class Packed2ZonesMaLayout  implements MaLayout {

    private final int z1NumCols, z1NumRows, z2NumCols, z2NumRows;
    private final float leftMargin;
    private final float rightMargin;
    private final float topMargin;
    private final float bottomMargin;
    private final float featureCellSizeX;
    private final float featureCellSizeY;


    public Packed2ZonesMaLayout(
            int z1NumCols, int z1NumRows,
            int z2NumCols, int z2NumRows,
            float leftMargin, float rightMargin,
            float topMargin, float bottomMargin,
            float featureCellSizeX, float featureCellSizeY ) {

        this.z1NumRows = z1NumRows;
        this.z1NumCols = z1NumCols;
        this.z2NumRows = z2NumRows;
        this.z2NumCols = z2NumCols;

        this.leftMargin = leftMargin;
        this.rightMargin = rightMargin;
        this.topMargin = topMargin;
        this.bottomMargin = bottomMargin;
        this.featureCellSizeX = featureCellSizeX;
        this.featureCellSizeY = featureCellSizeY;
    }


    public Object getScaled(float pixelSize) {
        Packed2ZonesMaLayout gml = new Packed2ZonesMaLayout(
            z1NumCols, z1NumRows,
            z2NumCols, z2NumRows,
            leftMargin/pixelSize, rightMargin/pixelSize,
            topMargin/pixelSize, bottomMargin/pixelSize,
            featureCellSizeX/pixelSize, featureCellSizeY/pixelSize);

        return gml;
    }


    public MaDimensions getDimensions() {
        return new MaDimensions(getWidth(), getHeight());
    }


    public float getWidth() {
        float xDim = Math.min((float)z1NumCols, (float)z2NumCols + 0.5f);
        return leftMargin + rightMargin + xDim * featureCellSizeX;
    }


    public float getHeight() {
        float yDim = Math.min((float)z1NumRows, (float)z2NumRows + 0.5f);
        return topMargin + bottomMargin + yDim * featureCellSizeY;
    }


    public Point2D getZoneLocation(int zoneIndex) {

        float x = leftMargin;
        float y = topMargin;
        if (zoneIndex != 0) {
            x += 0.5f*featureCellSizeX;
            y += 0.5f*featureCellSizeY;
        }

        return new Point2D.Float(x,y);
    }

}
