package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
/**
 *  In spirit, this class is an abstract class; subclasses must override
 *  either {@link #modifyValues} or {@link #getModifiedValue} methods.
 *  But it can't use the 'abstract' keyword because neither method must be
 *  overridden, but rather the subclass can choose which method to override.
 *
 *  If the channels are independent of each other, it's easier to override 
 *  {@link #modifyValues}.
 */
public class PointModifyFunction {

    // Use default constructor

    /**
     * Override this method if the values for different channels are not
     * independent of each other.  The values are modified in place.
     *
     * @param values  input and output array of channel values for a pixel.
     */
    public void modifyValues(float[] values) {
        for (int i=0; i<values.length; ++i) {
            values[i] = getModifiedValue(values[i]);
        }
    }


    /**
     * Override this method if the values for different channels are 
     * independent of each other. 
     *
     * @param srcValue  input source value for a channel in one pixel
     * @return  the modified intensity value for the input value
     */
    public float getModifiedValue(float srcValue) {
        throw new RuntimeException("Program bug: must override either " +
                "getModifiedValues or getModifiedValue.");
    }

}
