package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.Enumeration;

import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.StyledDocument;

import org.jdom.Document;
import org.jdom.input.SAXBuilder;


public class PropertiesListItem extends ExternalDataFactoryImpl
    implements ListItem {

    private final String name;
    boolean valid;
    String errorStr = null;
    String description = null;
    File xmlFile;
    String[] externalKeys;

    private DefaultStyledDocument doc;


    /*
     *   Assume properties file exists.
     */
    public PropertiesListItem(File propsFile, File xmlDir) {
        super();

        String xmlComment = null;
        String fName = propsFile.getName();
        int ind = fName.lastIndexOf('.');
        name = (ind > 0) ? fName.substring(0,ind) : fName;

        valid = true;

        try {
            Properties props = new Properties();
            props.load( new FileInputStream(propsFile) );
            putAll(props);
        }
        catch (java.io.IOException e) {
            errorStr = e.getMessage();
            valid = false;
        }

        if (valid) {
            String xmlFileName = (String)get("xmlFileName");
            if (xmlFileName == null) {
                errorStr = "Missing entry for 'xmlFileName'";
                valid = false;
            }
            else {
                try {
                    xmlFile = new File(xmlDir, xmlFileName);
                    Document xmlDoc =
                            XMLInspector.getDocument(xmlFile);
                    xmlComment = XMLInspector.getComment(xmlDoc);
                    externalKeys = XMLInspector.externalKeys(xmlDoc);
                }
                catch (Exception e) {
                    errorStr = "Problem parsing XML file: " +
                            e.getMessage();
                    valid = false;
                }

                doc = createDescription((String)get("comment"), xmlComment);
            }
        }
    }


    private DefaultStyledDocument createDescription(
            String propertyComment, String xmlComment) {
        DefaultStyledDocument sDoc = new DefaultStyledDocument();

        int offset = 0;

        try {
            String header1 = name + ":\n";
            sDoc.insertString(offset, header1, null);
            offset += header1.length();

            if (propertyComment != null) {
                sDoc.insertString(offset, propertyComment + "\n", null);
                offset += propertyComment.length() + 1;
            }

            String valuesStr = getPropertyValues();
            sDoc.insertString(offset, valuesStr, null);
            offset += valuesStr.length();

            String header2 = "\nXML:\n";
            sDoc.insertString(offset, header2, null);
            offset += header2.length();
    
            if (xmlComment != null) {
                sDoc.insertString(offset, xmlComment, null);
                offset += xmlComment.length();
            }
        }
        catch (javax.swing.text.BadLocationException e) {
            throw new RuntimeException("Shouldn't get here.");
        }

        return sDoc;
    }


    private String getPropertyValues( ) {
        StringBuffer buf = new StringBuffer();
        Enumeration keysEnum = keys();
        while (keysEnum.hasMoreElements()) {
            String key = (String)keysEnum.nextElement();
            buf.append(key + "=");
            buf.append((String)get(key) + "\n");
        }
        return buf.toString();
    }


    public Document getXMLDoc()
           throws org.jdom.JDOMException, java.io.IOException {
        return (xmlFile == null) ? null : new SAXBuilder().build(xmlFile);
    }


    public String[] externalKeys() {
        return externalKeys;
    }


    public File xmlFile() {
        return xmlFile;
    }



    //  Related to ListItem :

    public String toString() {
        return name;
    }


    public boolean isValid() {
        return valid;
    }


    public StyledDocument getDescription() {
        return doc;
    }


    public String errorString() {
        return errorStr;
    }

}
