package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.geom.Point2D;
import java.util.Random;
 
/**
 *  The encapsulation of parameters that effect the location of the spotter 
 *  tip center.
 */
public class SpotterTipLocationEffects implements SelfScaler {

    private float tipWobble = 0f;
    /** The standard deviation (in a normal distibution) of the deviation
        in position (radius along a random angle). */
    private float deltaX = 0f;
    /** Shift along the X axis in the tip position (for bent tip) */
    private float deltaY = 0f;
    /** Shift along the Y axis in the tip position (for bent tip) */

    private Random rand = null;


    public SpotterTipLocationEffects(float deltaX, float deltaY,
            float tipWobble) {
        this.deltaX = deltaX;
        this.deltaY = deltaY;
        this.tipWobble = tipWobble;
    }


    public void setRandom(Random rand) {
        this.rand = rand;
    }


    public Object getScaled(float pixelSize) {
       SpotterTipLocationEffects effects =
               new SpotterTipLocationEffects(deltaX/pixelSize, deltaY/pixelSize,
                       tipWobble/pixelSize);
       effects.setRandom(rand);

       return effects;
    }


    public Point2D getModifiedCenter(Point2D originalCenter) {
        if (rand == null) {
            rand = new Random();
        }

        float xCenter = (float)originalCenter.getX() + deltaX;
        float yCenter = (float)originalCenter.getY() + deltaY;
        if (tipWobble > 0f) {
            float wobbleRadius =
                    tipWobble * (float)Math.abs(rand.nextGaussian());
            float wobbleAngle = 2f * (float)(Math.PI * rand.nextDouble());
            xCenter += Math.cos(wobbleAngle) * wobbleRadius;
            yCenter += Math.sin(wobbleAngle) * wobbleRadius;
        }

        return new Point2D.Float(xCenter, yCenter);
    }

}
