package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.RenderingHints;
import java.awt.image.RenderedImage;
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.renderable.RenderedImageFactory;

import javax.media.jai.InterpolationBilinear;
import javax.media.jai.JAI;
import javax.media.jai.OperationDescriptor;
import javax.media.jai.OperationDescriptorImpl;
import javax.media.jai.OperationRegistry;
import javax.media.jai.ParameterListDescriptor;
import javax.media.jai.ParameterListDescriptorImpl;
import javax.media.jai.RenderedOp;
import javax.media.jai.registry.RIFRegistry;

/**
 *  Tilt : a JAI operation to rotate the entire image, gracefully dealing
 *  with edge effects.
 */
public class TiltOpDesc  extends OperationDescriptorImpl
                           implements RenderedImageFactory  {

    private static boolean isInitialized = false;
    public static final String OPERATION_NAME = "tilt";

    public static void init() {
        if (isInitialized)
            return;

        isInitialized = true;

        OperationRegistry opReg = 
                JAI.getDefaultInstance().getOperationRegistry();
        OperationDescriptor od = new TiltOpDesc();
                
        opReg.registerDescriptor(od);
        RIFRegistry.register(opReg, od.getName(), OPERATION_NAME,
                             (RenderedImageFactory)od);
    }


    public TiltOpDesc() {
        super(
                // resources
                new String[][] {
                     { "GlobalName",  OPERATION_NAME },
                     { "LocalName",  "Tilt" },
                     { "Vendor",  "Attotron Biosensor Corporation" },
                     { "Description",  "Constant image accepts MaImageLayout" },
                     { "DocURL",  "" },
                     { "Version",  "0.9" },
                     { "arg0Desc",  "tilt angle" },
                },

                // supported modes
                new String[] { "Rendered" },

                // source names
                null,   // use default naming convention "source0", ...

                // source classes
                new Class[][] { { RenderedImage.class } },

                new ParameterListDescriptorImpl(
                    // descriptor ??
                    new Object(),

                    // paramNames
                    new String[] { "tiltAngle" },

                    // paramClasses
                    new Class[] { Float.class },
                
                    // paramDefaults
                    new Object[] {
                            ParameterListDescriptor.NO_PARAMETER_DEFAULT },

                    // validParamValues
                    null
                )
        );

    }


    public int getNumSources() {
        return 1;
    }


    protected boolean validateSources(
            final String modeName,
            final ParameterBlock param,
            final StringBuffer message) {

        if (super.validateSources(modeName, param, message))  {
            return true;
        }
        
        return false;
    }


    protected boolean validateParameters(
            final String modeName,
            final ParameterBlock param,
            final StringBuffer message) {
        if (super.validateParameters(modeName, param, message))  {
            return true;
        }
        
        return false;
    }


    //  This is for RenderedImageFactory
    public RenderedImage create(final ParameterBlock params, 
                                final RenderingHints hints) {

        RenderedImage rim = (RenderedImage)params.getSource(0);
        float tiltAngle = params.getFloatParameter(0);

        float xCenter = (float)rim.getMinX() + (float)rim.getWidth()/2f;
        float yCenter = (float)rim.getMinY() + (float)rim.getHeight()/2f;

        ParameterBlock pb = new ParameterBlock();
        pb.addSource(rim);
        pb.add(xCenter);
        pb.add(yCenter);
        pb.add(tiltAngle);
        pb.add(new InterpolationBilinear());
        RenderedOp rop = JAI.create("rotate", pb);

        pb = new ParameterBlock();
        pb.addSource(rop);
        pb.add((float)rim.getMinX());
        pb.add((float)rim.getMinY());
        pb.add((float)rim.getWidth());
        pb.add((float)rim.getHeight());
        rop = JAI.create("crop", pb);

        rop.createInstance();
        return rop;
    }

}
