package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileReader;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JTextArea;


public class XMLChooser extends JFileChooser {

    //private JFileChooser fChooser;
    private DescriptionPanel descPanel;

    static public void main(String[] args) {
        JFrame jf = new JFrame();
        String currDirName = System.getProperty("user.dir");
        File currDir = new File(currDirName);
        XMLChooser xch = new XMLChooser(currDir);
        jf.setContentPane(xch);
        jf.show();
    }


    public XMLChooser() {
        this(new File(System.getProperty("user.dir")));
    }


    public XMLChooser(File startDir) {
        super(System.getProperty("user.dir"));
        DescriptionPanel descPanel = new DescriptionPanel();

        setAcceptAllFileFilterUsed(false);
        setFileFilter(new XMLFilter());

        setCurrentDirectory(startDir);
        setApproveButtonText("Select XML File");
        
        setAccessory(descPanel);
        addPropertyChangeListener(descPanel);
    }
    

    public class DescriptionPanel extends JTextArea 
            implements PropertyChangeListener {

        File currentDir;

        public void propertyChange(PropertyChangeEvent e) {
            Object source = e.getSource();
            JFileChooser jfc = null;
            if (source instanceof JFileChooser) {
                jfc = (JFileChooser)source;
                File selectedFile = jfc.getSelectedFile();
                if (selectedFile == null) {
                    setText("");
                }
                else {
                    displayFileInfo(selectedFile);
                }
                currentDir = jfc.getCurrentDirectory();
            }
        }

        private void displayFileInfo(File selectedFile) {
            String comment = XMLInspector.getDocComment(selectedFile);
            if (comment == null) {
                comment = "";
            }
            setText(comment);
        }


        private String getDescription(File descFile) {
    
            if (descFile.exists()) {
                char[] buffer = new char[2000];
                String fileText = null;
                try {
                    FileReader fr = new FileReader(descFile);
                    int numBytes = fr.read(buffer);
                    fileText = new String(buffer, 0, numBytes);
                }
                catch (Exception e) {
                    throw new RuntimeException("read error", e);
                }
                return fileText;
            }
            else {
                return "";
            }
        }
    }


    public class XMLFilter extends javax.swing.filechooser.FileFilter {

        public boolean accept(File path) {
            return (path.isDirectory() || path.getName().endsWith(".xml"));
        }

        public String getDescription() {
            return "XML files only";
        }

    }

}

