package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  This class is to handle arrays of non-atomic types.
 */
public class XMLHandlerArray extends XMLHandler {

    private final XMLHandler componentHandler;
    /** The handler for the type contained in the array */


    /**
     * Constructor, should only be called from {@link #getHandler} so as to
     * control the propagation of instances of handlers.
     * @param cls which Class to handle
     */
    protected XMLHandlerArray(Class cls, XMLHandler componentHandler)
            throws XMLUnhandledTypeException {
        super(cls);
        this.componentHandler = componentHandler;
    }


    protected boolean allowMultiple() { return true; }

    /** Allow data in the "defs" section, with later backward references ? */
    protected boolean allowBackwardRef()  {
        return componentHandler.allowBackwardRef();
    }

    /** Set up separate global typedef and always refer to it by type name ? */
    protected boolean needsTypedef() { return false; }



    /**
     * Given that the element already extracted from it's context, parse
     * the data (whose type should match that of the handler).
     *
     * @param elem  the JDOM element containing the data
     * @param altData  the background data context for parsing
     * @return  an object of the appropriate type
     */
    protected Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {
        return componentHandler.parseElement(elem, altData);
    }



    /**
     *
     */
    public String xmlTypeName() {
        return componentHandler.xmlTypeName();
        //return null;
    }

}
