package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 * The base class for most custom handler extensions.
 */
public abstract class XMLHandlerStd extends XMLHandler {
    private final boolean allowBackwardRef;


    public XMLHandlerStd(Class cls, boolean allowBackwardRef) {
        super(cls);
        this.allowBackwardRef = allowBackwardRef;
    }


    public XMLHandlerStd(Class cls) {
        this(cls, false);
    }

    /**
     * Generate Schema type definition for the class (using JDOM).
     *
     * @return a JDOM Element with schema for type
     */
    protected abstract Element schemaTypedef() throws XMLUnhandledTypeException;


    /**
     * Given that the element already extracted from it's context, parse
     * the data (whose type should match that of the handler).
     *
     * @param elem  the JDOM element containing the data
     * @param altData  the background data context for parsing
     * @return  an object of the appropriate type
     */
    protected abstract Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException;


    /**
     * Return the label to be used in the name of schema type defs.
     *
     * @return schema type label (or null if no data type name is to be used)
     */
    public String xmlTypeName() {
        String name = handledClass().getName();
        int dotPos = name.lastIndexOf(".");
        return (dotPos == -1) ? name : name.substring(dotPos+1);
    }


    protected boolean allowBackwardRef()  { return allowBackwardRef; }
    /** Allow data in the "defs" section, with later backward references ? */

    final protected boolean allowMultiple()  { return false; }
    /** Allow multiple elements for this data type ? */

    final protected boolean needsTypedef() { return true; };
    /** Does a schema typedef need to be generated for this data type ? */
}
