package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.util.HashMap;
import java.util.Iterator;

import javax.media.jai.BorderExtender;
import javax.media.jai.BorderExtenderConstant;

import org.jdom.Element;

/**
 *  XML wrapper for BorderExtender.
 */
public class X_BorderExtender extends XMLHandlerStd {

    public X_BorderExtender()  throws XMLUnhandledTypeException {
        super(BorderExtender.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is used in some JAI operations " +
                "to control how border issues are handled.";
    }


    static HashMap typeMap = new HashMap();
    static {
        typeMap.put("Copy",
                BorderExtender.createInstance(BorderExtender.BORDER_COPY));
        typeMap.put("Reflect", 
                BorderExtender.createInstance(BorderExtender.BORDER_REFLECT));
        typeMap.put("Wrap",
                BorderExtender.createInstance(BorderExtender.BORDER_WRAP));
        typeMap.put("Zero",
                BorderExtender.createInstance(BorderExtender.BORDER_ZERO));
        typeMap.put("One", new BorderExtenderConstant(new double[] {1.0}));
    }


    static private Element enumElem(String value) {
        Element elem = XMLUtils.xsd("enumeration");
        elem.setAttribute("value", value);
        return elem;
    }


    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * BorderExtender.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {

        Element restrictElem = XMLUtils.xsd("restriction");
        restrictElem.setAttribute("base", "xsd:token");
        Iterator iter = typeMap.keySet().iterator();
        while (iter.hasNext()) {
            restrictElem.addContent(enumElem((String)iter.next()));
        }

        return XMLUtils.xsdSimpleType(xmlTypeName(), restrictElem);
    }


    /**
     * Given a jdom Element object (with children), construct a BorderExtender
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated BorderExtender object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        String borderType = elem.getTextTrim();

        BorderExtender bExt = (BorderExtender)typeMap.get(borderType);

        if (bExt == null) {
            throw new XMLParseException("", elem);
        }

        return bExt;
    }

}
