package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.util.Random;

import org.jdom.Element;
import org.jdom.filter.ElementFilter;
import org.jdom.filter.Filter;

/**
 *  XML wrapper for CircleFeatureShapeFactory.
 *
 *  A flavor of FeatureShapeFactory that produces circular features,
 *  including support for the variability of location, size, and shape.
 *
 *  Note that all random effects in this class are related to array 
 *  manufacture -- therefore in the Cybertory, the random seed should be
 *  based on the serial number.
 */
public class X_CircleFeatureShapeFactory extends XMLHandlerStd {

    public X_CircleFeatureShapeFactory() throws XMLUnhandledTypeException {
        super(CircleFeatureShapeFactory.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " generates nominally circular " +
                "shapes to be used in painting features.";
    }


    private XMLChildFilter randFilter = null;
    private XMLChildFilter abFilter = null;
    private Filter roundedFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {

        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] { 
                     getHandler(Random.class).schemaInContext(false, false),
                     XMLUtils.xsdElement("locationVariability", "xsd:float",
                       false),
                     XMLUtils.xsdElement("sizeVariability", "xsd:float", false),
                     XMLUtils.xsdElement("shapeVariability", "xsd:int", false),
                     XMLUtils.xsdElement("roundedCorners", "xsd:boolean",
                       false),
                     getHandler(AngleBias[].class).schemaInContext(false, false)
                     } ));
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        CircleFeatureShapeFactory factory = new CircleFeatureShapeFactory();

        if (randFilter == null) {
            randFilter = getHandler(Random.class).getXMLChildFilter(false);
        }
        Random rand = (Random)getHandler(Random.class).parseChildren(
                elem, randFilter, altData, false);

        if (rand == null) {
            rand = new Random();
        }
                
        float locationVariability =
                XMLUtils.getChildFloat(elem, "locationVariability", 0f);
        float sizeVariability =
                XMLUtils.getChildFloat(elem, "sizeVariability", 0f);
        int shapeVariability = 
                XMLUtils.getChildInteger(elem, "shapeVariability", 0);
        if (shapeVariability > 7) {
            shapeVariability = 7;
        }
        
        if (roundedFilter == null) {
            roundedFilter = new ElementFilter("roundedCorners");
        }
        Boolean oRoundedCorners = (Boolean)
                getHandler(Boolean.class).parseChildren(elem,
                roundedFilter, altData, false);

        boolean roundedCorners = (oRoundedCorners == null) ? false :
                oRoundedCorners.booleanValue();

        if (abFilter == null) {
            abFilter = getHandler(AngleBias[].class).getXMLChildFilter(false);
        }
        AngleBias[] biases = (AngleBias[])
                getHandler(AngleBias[].class).parseChildren(elem,
                abFilter, altData, false);

        factory.setVariability(rand, locationVariability, sizeVariability,
                shapeVariability, roundedCorners, biases);

        return factory;
    }

}
