package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for FImageLayout.
 */
public class X_FImageLayout extends XMLHandlerStd {

    public X_FImageLayout()  throws XMLUnhandledTypeException {
        super(FImageLayout.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " a variation on ImageLayout " +
                "that uses 'float' values and is scalable for different " +
                "resolutions.";
    }


    private XMLChildFilter filter = null;


    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * FImageLayout.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        Element minXElem = XMLUtils.xsdElement("minX", "xsd:float", false);
        minXElem.setAttribute("default", "0");
        Element minYElem = XMLUtils.xsdElement("minY", "xsd:float", false);
        minYElem.setAttribute("default", "0");
        
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("choice", new Element[] {
                  XMLUtils.xsdGrouping("sequence", new Element[] {
                    minXElem,
                    minYElem,
                    XMLUtils.xsdElement("width", "xsd:float"),
                    XMLUtils.xsdElement("height", "xsd:float")
                    } ),
                  getHandler(MaLayout.class).schemaInContext(false, false) })
                );
    }


    /**
     * Given a jdom Element object (with children), construct a FImageLayout
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated FImageLayout object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (filter == null) {
            filter = getHandler(MaLayout.class).getXMLChildFilter(false);
        }

        MaLayout maLayout = (MaLayout)
                getHandler(MaLayout.class).parseChildren(elem, filter,
                altData, false);

        return (maLayout != null) ?
                new FImageLayout(0f, 0f, maLayout.getWidth(),
                        maLayout.getHeight())   :
                new FImageLayout(
                        XMLUtils.getChildFloat(elem, "minX", 0f),
                        XMLUtils.getChildFloat(elem, "minY", 0f),
                        XMLUtils.getChildFloat(elem, "width"),
                        XMLUtils.getChildFloat(elem, "height") );
    }

}
