package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for FeatureCharacteristics.
 */
public class X_FeatureCharacteristics extends XMLHandlerStd {

    public X_FeatureCharacteristics()  throws XMLUnhandledTypeException {
        super(FeatureCharacteristics.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " encapsulates characteristics " +
                "for a specific feature.";
    }


    private XMLChildFilter factoryFilter = null;
    private XMLChildFilter textureFilter = null;

    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * FeatureCharacteristics.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdElement("row", "xsd:positiveInteger"),
                  XMLUtils.xsdElement("col", "xsd:positiveInteger"),
                  getHandler(FeatureShapeFactory.class).schemaInContext(false,
                          false),
                  getHandler(FeatureTexture.class).schemaInContext(false, 
                          false) } ) );
    }


    /**
     * Given a jdom Element object (with children), construct a
     * FeatureCharacteristics instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated FeatureCharacteristics object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        int row = XMLUtils.getChildInteger(elem, "row");
        int col = XMLUtils.getChildInteger(elem, "col");

        if (factoryFilter == null) {
            factoryFilter = getHandler(FeatureShapeFactory.class
                    ).getXMLChildFilter(false);
        }
        FeatureShapeFactory factory = (FeatureShapeFactory)
                getHandler(FeatureShapeFactory.class).parseChildren(elem,
                        factoryFilter, altData, false);
                        
        if (textureFilter == null) {
            textureFilter =
                    getHandler(FeatureTexture.class).getXMLChildFilter(false);
        }
        FeatureTexture texture = (FeatureTexture)
                getHandler(FeatureTexture.class).parseChildren(elem,
                        textureFilter, altData, false);

        return new FeatureCharacteristics(row, col, factory, texture);
    }

}
