package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for FeaturesSpecs.
 */
public class X_FeaturesSpecs extends XMLHandlerStd {

    public X_FeaturesSpecs()  throws XMLUnhandledTypeException {
        super(FeaturesSpecs.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " contains general " +
                "specifications of features that are global to the array.";
    }


    private XMLChildFilter factoryFilter = null;
    private XMLChildFilter textureFilter = null;

    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * FeaturesSpecs.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdElement("featureSize", "xsd:float"),
                  XMLUtils.xsdElement("featureCellSizeX", "xsd:float"),
                  XMLUtils.xsdElement("featureCellSizeY", "xsd:float"),
                  getHandler(FeatureShapeFactory.class).schemaInContext(
                    "defaultShapeFactory", false, false),
                  getHandler(FeatureTexture.class).schemaInContext(
                    "defaultTexture", false, false) } ) );
    }


    /**
     * Given a jdom Element object (with children), construct a FeaturesSpecs
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated FeaturesSpecs object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        float featureSize = XMLUtils.getChildFloat(elem, "featureSize");
        float featureCellSizeX =
                XMLUtils.getChildFloat(elem, "featureCellSizeX");
        float featureCellSizeY =
                XMLUtils.getChildFloat(elem, "featureCellSizeY");

        if (factoryFilter == null) {
            factoryFilter = getHandler(FeatureShapeFactory.class
                    ).getXMLChildFilter("defaultShapeFactory", false);
        }
        FeatureShapeFactory defaultShapeFactory = (FeatureShapeFactory)
                getHandler(FeatureShapeFactory.class).parseChildren(elem,
                        factoryFilter, altData, false);

        if (textureFilter == null) {
            textureFilter = getHandler(FeatureTexture.class).getXMLChildFilter(
                    "defaultTexture", false);
        }
        FeatureTexture defaultTexture = (FeatureTexture)
                getHandler(FeatureTexture.class).parseChildren(elem,
                        textureFilter, altData, false);

        return new FeaturesSpecs(featureSize,
                featureCellSizeX, featureCellSizeY,
                defaultShapeFactory, defaultTexture);
    }

}
