package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.io.File;

import org.jdom.Element;

/**
 *  XML wrapper for File.
 */
public class X_File extends XMLHandlerStd {

    public X_File() {
        super(File.class);
    }


    public String typeDescription() {
        return "Java's standard " + this.getClass().getName() + " class.";
    }


    private XMLChildFilter nameFilter = null;
    private XMLChildFilter directoryFilter = null;


    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * FeaturesSpecs.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  getHandler(String.class).schemaInContext("directory",
                    false, true),
                  getHandler(String.class).schemaInContext("fileName",
                    true, true) } ) );
    }


    /**
     * Given a jdom Element object (with children), construct a FeaturesSpecs
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated FeaturesSpecs object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (nameFilter == null) {
            nameFilter = getXMLChildFilter("fileName", true);
        }
        String name = (String)getHandler(String.class).parseChildren(
                elem, nameFilter, altData, true);

        if (directoryFilter == null) {
            directoryFilter = getXMLChildFilter("directory", true);
        }
        String directory = (String)getHandler(String.class).parseChildren(
                elem, directoryFilter, altData, false);
         
        File file = (directory != null) ? new File(directory, name) :
                new File(name);

        return file;
    }

}
