package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

public class X_GradientFunction  extends XMLHandlerStd {

    public X_GradientFunction()  throws XMLUnhandledTypeException {
        super(GradientFunction.class, false);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is an ImageFunction for " +
                "creating a linear intensity gradient, useful for " +
                "background effects.";
    }


    /**
     * Produces an 'xsd:element' component of an XML schema for the
     * GradientFunction.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdElement("gradientAngle", "xsd:float"),
                  XMLUtils.xsdElement("centerIntensity", "xsd:float"),
                  XMLUtils.xsdElement("rateOfIntensityChange", "xsd:float")
                  } ) );
    }


    /**
     * Given a jdom Element object (with children), construct a FeaturesSpecs
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated FeaturesSpecs object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        float gradientAngle = XMLUtils.getChildFloat(elem, "gradientAngle");
        float centerIntensity = XMLUtils.getChildFloat(elem, "centerIntensity");
        float rateOfIntensityChange =
                XMLUtils.getChildFloat(elem, "rateOfIntensityChange");

        return new GradientFunction(gradientAngle, centerIntensity,
                rateOfIntensityChange);
    }
}
