package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;
import org.jdom.filter.Filter;

/**
 *  XML wrapper for GrayscaleTiffs.
 */
public class X_GrayscaleTiffs extends XMLHandlerStd {

    public X_GrayscaleTiffs() {
        super(GrayscaleTiffs.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is an image file save type " +
                "that stores one channel per file, with integers for pixel " +
                "values.";
    }


    private Filter nbFilter = null;

    public Element schemaTypedef() throws XMLUnhandledTypeException {
        Element seqElem = XMLUtils.xsd("sequence");
        X_CommonScanSaveParams.addSchemaCommonParams(seqElem, 
                GrayscaleTiffs.defaultFilenameExtension);

        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  seqElem,
                  getHandler(Integer.class).schemaInContext("numBits",
                          false, true) } ));
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (nbFilter == null) {
            nbFilter = getHandler(Integer.class).getXMLChildFilter("numBits",
                    true);
        }
        
        Integer numBitsInt = (Integer)
            getHandler(Integer.class).parseChildren(elem, nbFilter, altData, 
                    false);

        int numBits = (numBitsInt != null) ? numBitsInt.intValue() : 16;

        CommonScanSaveParams commonParams =
                X_CommonScanSaveParams.parseCommonParams(elem,
                GrayscaleTiffs.defaultFilenameExtension, altData);

        return new GrayscaleTiffs(commonParams, numBits);
    }

}
