package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.Dimension;

import org.jdom.Element;

/**
 *  XML wrapper for GridMaLayout.
 *
 * This class encapsulates the layout of a microarray with zones on a uniform
 * grid, and where each zone has the same number of rows and columns.
 */
public class X_GridMaLayout extends XMLHandlerStd {

    public X_GridMaLayout() throws XMLUnhandledTypeException {
        super(GridMaLayout.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " encapsulates the layout of a " +
                "microarray with zones on a uniform grid, and where each " +
                "zone has the same number of rows and columns.";
    }


    private XMLChildFilter fsFilter = null;
    private XMLChildFilter gfFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdElement("leftMargin", "xsd:float"),
                  XMLUtils.xsdElement("rightMargin", "xsd:float"),
                  XMLUtils.xsdElement("topMargin", "xsd:float"),
                  XMLUtils.xsdElement("bottomMargin", "xsd:float"),
                  XMLUtils.xsdElement("gapBetweenZonesX", "xsd:float"),
                  XMLUtils.xsdElement("gapBetweenZonesY", "xsd:float"),
                  XMLUtils.xsdGrouping("choice", new Element[] {
                    XMLUtils.xsdGrouping("sequence", new Element[] {
                      XMLUtils.xsdElement("featureCellSizeX", "xsd:float"),
                      XMLUtils.xsdElement("featureCellSizeY", "xsd:float") } ),
                    getHandler(FeaturesSpecs.class).schemaInContext(true,
                            false) }),
                  XMLUtils.xsdGrouping("choice", new Element[] {
                    getHandler(GridFeaturesIntensities.class).schemaInContext(
                            true, false),
                    XMLUtils.xsdGrouping("sequence", new Element[] {
                      XMLUtils.xsdElement("numMetacols", "xsd:int"),
                      XMLUtils.xsdElement("numMetarows", "xsd:int"),
                      XMLUtils.xsdElement("numCols", "xsd:int"),
                      XMLUtils.xsdElement("numRows", "xsd:int") } ), } )
                } ));
    }


    public Object parseElement(Element elem, XMLAltData altData) 
            throws XMLParseException, XMLUnhandledTypeException {
        float leftMargin = XMLUtils.getChildFloat(elem, "leftMargin");
        float rightMargin = XMLUtils.getChildFloat(elem, "rightMargin");
        float topMargin = XMLUtils.getChildFloat(elem, "topMargin");
        float bottomMargin = XMLUtils.getChildFloat(elem, "bottomMargin");
        float gapBetweenZonesX =
                XMLUtils.getChildFloat(elem, "gapBetweenZonesX");
        float gapBetweenZonesY =
                XMLUtils.getChildFloat(elem, "gapBetweenZonesY");

        if (fsFilter == null) {
            fsFilter = getHandler(FeaturesSpecs.class).getXMLChildFilter(false);
        }
        FeaturesSpecs fs = (FeaturesSpecs)
                getHandler(FeaturesSpecs.class).parseChildren(elem,
                        fsFilter, altData, false);

        float featureCellSizeX, featureCellSizeY;
        if (fs != null) {
            featureCellSizeX = fs.featureCellSizeX;
            featureCellSizeY = fs.featureCellSizeY;
        }
        else {
            featureCellSizeX = XMLUtils.getChildFloat(elem, "featureCellSizeX");
            featureCellSizeY = XMLUtils.getChildFloat(elem, "featureCellSizeY");
        }
         

        int numMetacols = -1;
        int numMetarows = -1;
        int numCols = -1;
        int numRows = -1;

        if (gfFilter == null) {
            gfFilter = getHandler(GridFeaturesIntensities.class
                    ).getXMLChildFilter(false);
        }
        GridFeaturesIntensities gfi = (GridFeaturesIntensities)
                getHandler(GridFeaturesIntensities.class).parseChildren(elem,
                        gfFilter, altData, false);
        if (gfi != null) {
             Dimension metaDim = gfi.getMetaDimension();
             numMetacols = metaDim.width;
             numMetarows = metaDim.height;
             Dimension zDim = gfi.getZoneDimension();
             numCols = zDim.width;
             numRows = zDim.height;
        }
        else {
            numMetacols = XMLUtils.getChildInteger(elem, "numMetacols");
            numMetarows = XMLUtils.getChildInteger(elem, "numMetarows");
            numCols = XMLUtils.getChildInteger(elem, "numCols");
            numRows = XMLUtils.getChildInteger(elem, "numRows");
        }
 
        GridMaLayout gLayout = new GridMaLayout(numMetacols, numMetarows,
                numCols, numRows,
                leftMargin, rightMargin, topMargin, bottomMargin,
                gapBetweenZonesX, gapBetweenZonesY,
                featureCellSizeX, featureCellSizeY);

        if (gfi != null) {
            gLayout.addZonesGridLocations(gfi.getZoneMetaCoordinates());
        }

        return gLayout;
    }

}
