package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for ImageFunction instantiated from a class name.
 */
public class X_IFunctionFromName extends XMLHandlerStd {

    public X_IFunctionFromName()  throws XMLUnhandledTypeException {
        super(IFunctionFromName.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is a  class for ImageFunction " +
                "implementations that are instantiated using a no-parameter " +
                "constructor.";
    }


    private XMLChildFilter cnFilter = null;

   /**
     * Produces an 'xsd:element' component of an XML schema for the
     * FeaturesSpecs.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  getHandler(String.class).schemaInContext("className", true,
                          true) } ) );
    }


    /**
     * Given a jdom Element object (with children), construct a FeaturesSpecs
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated FeaturesSpecs object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (cnFilter == null) {
            cnFilter = getHandler(String.class).getXMLChildFilter("className",
                    true);
        }
        String className = (String)getHandler(String.class).parseChildren(elem,
                cnFilter, altData, true);
        
        try {
            return new IFunctionFromName(className);
        }
        catch (Exception e) {
            throw new XMLParseException("Error instantiating image function"
                    + className, elem, e);
        }
    }

}


