package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for NumberOfChannels.
 */
public class X_NumberOfChannels extends XMLHandlerStd {

    public X_NumberOfChannels()  throws XMLUnhandledTypeException {
        super(NumberOfChannels.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is an encapsulation of the " +
                "number of channels to be in the image.";
    }


    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * NumberOfChannels.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        Element enumElem = XMLUtils.xsd("enumeration");
        enumElem.setAttribute("value", "");

        Element restrElem = XMLUtils.xsd("restriction", enumElem);
        restrElem.setAttribute("base", "xsd:token");

        Element unionElem = XMLUtils.xsd("union");
        unionElem.setAttribute("memberTypes", "xsd:positiveInteger");
        unionElem.addContent(XMLUtils.xsd("simpleType", restrElem));
        
        return XMLUtils.xsdSimpleType(xmlTypeName(), unionElem);
    }


    /**
     * Given a jdom Element object (with children), construct a NumberOfChannels
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated NumberOfChannels object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {
        String numChStr = elem.getTextTrim();
        try {
            int numCh = (numChStr != null && numChStr.length() > 0) ?
                    Integer.parseInt(numChStr) : altData.numberOfChannels;
            return new NumberOfChannels(numCh);
        }
        catch (NumberFormatException e) {
            throw new XMLFormatException(elem, e);
        }
    }

}
