package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.util.HashMap;

import javax.media.jai.OperationDescriptorImpl;
import javax.media.jai.ParameterListDescriptor;

import org.jdom.Element;

public class X_OperationJAI {

    static private final String sourceLabel = "source";
    static private HashMap jaiOps = new HashMap();
            /** A look-up table of X_OperationJAI instances */

    static XMLChildFilter sourceFilter = null;

    protected final String opName;
    private ParameterListDescriptor paramsDesc;

    private final int numSources;
    private X_JAIParam[] params;
    private HashMap paramMap = new HashMap();

    private final String description;


    public X_OperationJAI(OperationDescriptorImpl opDesc)
            throws XMLUnhandledTypeException {
        opName = opDesc.getName().toLowerCase();
        jaiOps.put(opName, this);

        description = opDesc.getResourceBundle(null).getString("Description");
        numSources = opDesc.getNumSources();
        paramsDesc = opDesc.getParameterListDescriptor("Rendered");
        int numParams = paramsDesc.getNumParameters();
        params = new X_JAIParam[numParams];
        for (int i=0; i<numParams; ++i) {
            params[i] = new X_JAIParam(i, paramsDesc);
            paramMap.put(params[i].getName(), params[i]);
        }
    }


    /**
     * Generate the schema for this particular schema.
     * 
     * @return a Element object describing this operation's schema
     */
    protected Element opSchema() throws XMLUnhandledTypeException {
        Element seqElem = XMLUtils.xsdGrouping("sequence", null);
        
        for (int i=0; i<params.length; ++i) {
            if (!params[i].hide()) {
                seqElem.addContent(params[i].schemaParam());
            }
        }

        if (numSources > 0) {
            XMLHandler handler =
                    XMLHandler.getHandler(PendingRenderedOp[].class);
            Element srcsElem =
                    handler.schemaInContext(sourceLabel, true, false);

            // zero can be used to represent flexible num of sources
            srcsElem.setAttribute("minOccurs", String.valueOf(numSources));
            if (numSources > 0) {
                srcsElem.setAttribute("maxOccurs", String.valueOf(numSources));
            }
            seqElem.addContent(srcsElem);
        }

        Element topElem = XMLUtils.xsdElement(opName);
        if (description != null) {
            topElem.addContent(XMLUtils.xsdDocumentation(description));
        }
        topElem.addContent(XMLUtils.xsdComplexType(null, seqElem));

        return topElem;
    }


    public PendingParameterBlockJAI parseOp(Element elem,
            XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        XMLHandler handler;
        PendingParameterBlockJAI pb = new PendingParameterBlockJAI(opName);

        for (int i=0; i<params.length; ++i) {
            X_JAIParam param = params[i];
            Object obj = param.parseParam(elem, altData);
            pb.setParameter(param.getName(), obj);
        }

        boolean required = (numSources > 0);
        handler = XMLHandler.getHandler(PendingRenderedOp[].class);
        if (sourceFilter == null) {
            sourceFilter = new XMLChildFilter(sourceLabel, false, true);
        }
        PendingRenderedOp[] sources = (PendingRenderedOp[])
                handler.parseChildren(elem, sourceFilter, altData, required);
        if (sources != null) {
            for (int i=0; i<sources.length; ++i) {
                pb.addSource(sources[i]);
            }
        }
        
        return pb;
    }


    ////////////////////////////////////////////////////////////////////////
    // These methods are pass-thru to modify X_JAIParam fields.
    ////////////////////////////////////////////////////////////////////////
    
    public void setRequired(String paramName, boolean required) {
        ((X_JAIParam)paramMap.get(paramName)).setRequired(required);
    }

    public void setHide(String paramName, boolean hide) {
        ((X_JAIParam)paramMap.get(paramName)).setHide(hide);
    }

    public void setAllowExternalData(String paramName,
            boolean allowExternalData) {
        ((X_JAIParam)paramMap.get(paramName)).setAllowExternalData(
                allowExternalData);
    }

    public void setDefaultValue(String paramName, Object paramDefault) {
        ((X_JAIParam)paramMap.get(paramName)).setDefaultValue(paramDefault);
    }



    /**
     *  This class encapsulates information about parameter to JAI operation.
     */
    public class X_JAIParam {
        
        private final String paramName;
        private final Class paramClass;
        private Object paramDefault = null;
    
        private boolean hide = false;
        private boolean required = false;
        private boolean allowExternalData = false;
        //private boolean allowBackwardRef = false;
        private XMLChildFilter filter = null;
        private XMLHandler handler = null;
       
    
        public X_JAIParam(int paramIndex, ParameterListDescriptor paramsDesc)
                throws XMLUnhandledTypeException {
            this(paramsDesc.getParamNames()[paramIndex],
                 paramsDesc.getParamClasses()[paramIndex],
                 paramsDesc.getParamDefaults()[paramIndex]);
        }
    
    
        public X_JAIParam(String paramName, Class paramClass,
                Object paramDefault)
                throws XMLUnhandledTypeException {
            this.paramName = paramName;
            this.paramClass = paramClass;
            this.paramDefault = paramDefault;

            required = (paramDefault ==
                    ParameterListDescriptor.NO_PARAMETER_DEFAULT);
        }


        public String getName() {
            return paramName;
        }


        public Element schemaParam() throws XMLUnhandledTypeException {
            if (handler == null) {
                handler = XMLHandler.getHandler(paramClass);
                if (handler == null) {
                    throw new XMLUnhandledTypeException(paramClass);
                }
            }
            return handler.schemaInContext(paramName, required,
                    allowExternalData);
        }
    

        public Object parseParam(Element elem, XMLAltData altData)
                throws XMLParseException, XMLUnhandledTypeException {
            if (hide) {
                return paramDefault;
            }

            if (handler == null) {
                handler = XMLHandler.getHandler(paramClass);
                if (handler == null) {
                    throw new XMLUnhandledTypeException(paramClass);
                }
            }
            if (filter == null) {
                filter = handler.getXMLChildFilter(paramName,
                        allowExternalData);
            }
            return handler.parseChildren(elem, filter, altData, required);
        }

    
        public void setRequired(boolean required) {
            this.required = required;
            if (required) {
                hide = false;
            }
        }
    
    
        public void setHide(boolean hide) {
            this.hide = hide;
            if (required) {
                throw new RuntimeException("*******");
            }
        }
    
    
        public boolean hide() {
            return hide;
        }
    
    
        public void setDefaultValue(Object paramDefault) {
            this.paramDefault = paramDefault;
        }
    
    
        public void setAllowExternalData(boolean allowExternalData) {
            this.allowExternalData = allowExternalData;
        }
    
    
/*
        public void setAllowBackwardRef(boolean allowBackwardRef) {
            this.allowBackwardRef = allowBackwardRef;
        }
*/
    }
}
