package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;
//import java.awt.Dimension;


/**
 *  XML wrapper for Packed2ZonesMaLayout.
 *
 * This class encapsulates the layout of a microarray with zones on a uniform
 * grid, and where each zone has the same number of rows and columns.
 */
public class X_Packed2ZonesMaLayout extends XMLHandlerStd {

    public X_Packed2ZonesMaLayout() throws XMLUnhandledTypeException {
        super(Packed2ZonesMaLayout.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " encapsulates the layout of a " +
                "microarray with 2 overlayed packed zones.";
    }


    private XMLChildFilter fsFilter = null;
    private XMLChildFilter fiFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdElement("leftMargin", "xsd:float"),
                  XMLUtils.xsdElement("rightMargin", "xsd:float"),
                  XMLUtils.xsdElement("topMargin", "xsd:float"),
                  XMLUtils.xsdElement("bottomMargin", "xsd:float"),
                  XMLUtils.xsdGrouping("choice", new Element[] {
                    XMLUtils.xsdGrouping("sequence", new Element[] {
                      XMLUtils.xsdElement("featureCellSizeX", "xsd:float"),
                      XMLUtils.xsdElement("featureCellSizeY", "xsd:float") } ),
                    getHandler(FeaturesSpecs.class).schemaInContext(true,
                            false) }),
                  XMLUtils.xsdGrouping("choice", new Element[] {
                    getHandler(FeaturesIntensities.class).schemaInContext(
                            true, false),
                    XMLUtils.xsdGrouping("sequence", new Element[] {
                      XMLUtils.xsdElement("z1NumCols", "xsd:int"),
                      XMLUtils.xsdElement("z1NumRows", "xsd:int"),
                      XMLUtils.xsdElement("z2NumCols", "xsd:int"),
                      XMLUtils.xsdElement("z2NumRows", "xsd:int") } ), } )
                } ));
    }


    public Object parseElement(Element elem, XMLAltData altData) 
            throws XMLParseException, XMLUnhandledTypeException {
        float leftMargin = XMLUtils.getChildFloat(elem, "leftMargin");
        float rightMargin = XMLUtils.getChildFloat(elem, "rightMargin");
        float topMargin = XMLUtils.getChildFloat(elem, "topMargin");
        float bottomMargin = XMLUtils.getChildFloat(elem, "bottomMargin");

        if (fsFilter == null) {
            fsFilter = getHandler(FeaturesSpecs.class).getXMLChildFilter(false);
        }
        FeaturesSpecs fs = (FeaturesSpecs)
                getHandler(FeaturesSpecs.class).parseChildren(elem,
                        fsFilter, altData, false);

        float featureCellSizeX, featureCellSizeY;
        if (fs != null) {
            featureCellSizeX = fs.featureCellSizeX;
            featureCellSizeY = fs.featureCellSizeY;
        }
        else {
            featureCellSizeX = XMLUtils.getChildFloat(elem, "featureCellSizeX");
            featureCellSizeY = XMLUtils.getChildFloat(elem, "featureCellSizeY");
        }
         

        int z1NumCols, z1NumRows, z2NumCols, z2NumRows;

        if (fiFilter == null) {
            fiFilter = getHandler(FeaturesIntensities.class
                    ).getXMLChildFilter(false);
        }
        FeaturesIntensities fi = (FeaturesIntensities)
                getHandler(FeaturesIntensities.class).parseChildren(elem,
                        fiFilter, altData, false);
        if (fi != null) {

            float[][][] z1 = fi.intensitiesForZone(0);
            z1NumRows = z1.length;
            z1NumCols = 0;
            for (int row=0; row<z1.length; ++row) {
                if (z1[row].length > z1NumCols)
                z1NumCols = z1[row].length;
            }


            float[][][] z2 = fi.intensitiesForZone(1);
            z2NumRows = z2.length;
            z2NumCols = 0;
            for (int row=0; row<z2.length; ++row) {
                if (z2[row].length > z2NumCols)
                z2NumCols = z2[row].length;
            }
        }
        else {
            z1NumCols = XMLUtils.getChildInteger(elem, "z1NumCols");
            z1NumRows = XMLUtils.getChildInteger(elem, "z1NumRows");
            z2NumCols = XMLUtils.getChildInteger(elem, "z2NumCols");
            z2NumRows = XMLUtils.getChildInteger(elem, "z2NumRows");
        }
 
        Packed2ZonesMaLayout pLayout = new Packed2ZonesMaLayout(
                z1NumCols, z1NumRows, z2NumCols, z2NumRows,
                leftMargin, rightMargin, topMargin, bottomMargin,
                featureCellSizeX, featureCellSizeY);

        return pLayout;
    }

}
