package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.jdom.Attribute;
import org.jdom.DataConversionException;
import org.jdom.Element;
import org.jdom.filter.ElementFilter;
import org.jdom.filter.Filter;

/**
 *  XML wrapper for PendingRenderedOp.
 */
public class X_PendingRenderedOp extends XMLHandlerStd {

    public X_PendingRenderedOp() {
        super(PendingRenderedOp.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is analogous to " +
                "javax.media.jai.RenderedOp, but in addition it handles " +
                "recursive application of pixel size. " +
                "Once the pixel size is known, the manifestResolution " +
                "method is called to create a javax.media.jai.RenderedOp " +
                "with the appropriate pixel size.";
    }


    static private HashMap jaiOps = new HashMap(); 
    /** Map of X_OperationJAI for JAI operations, keyed by (short)
        operation name (in lower case) */

    static Filter opFilter = null;
    static Filter numChFilter = null;

    static public void registerOperation(X_OperationJAI opJAI) {
        jaiOps.put(opJAI.opName, opJAI);
    }


    static public String[] getOpNames() {
        Set keys = jaiOps.keySet();
        return (String[])keys.toArray(new String[keys.size()]);
    }


    /**
     *  Generate the schema as a choice of elements, each element for
     *  one operation (PendingRenderedOp instance).
     *
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        // Operations:
        Element opChoice = XMLUtils.xsd("choice");
        String[] opNames = getOpNames();
        Arrays.sort(opNames);
        for (int i=0; i<opNames.length; ++i) {
            String opName = opNames[i];
            X_OperationJAI op = (X_OperationJAI)jaiOps.get(opName);
            opChoice.addContent( op.opSchema() );
        }

        Element srcAttr =
                XMLUtils.xsdAttribute("sourceIndex", "xsd:nonNegativeInteger");

        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  opChoice,
                  XMLUtils.xsdGrouping("sequence", new Element[] {
                    XMLUtils.xsdGrouping("choice", new Element[] {
                      getHandler(FImageLayout.class).schemaInContext(false,false),
                      XMLUtils.xsdElement("propagateImageLayout", 
                        XMLUtils.xsdComplexType(null, srcAttr))
                      } ),
                    getHandler(NumberOfChannels.class).schemaInContext(
                            false, false),
                    }, false )  } ) );
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {
        if (opFilter == null) {
            opFilter = new Filter() {
                public boolean matches(Object obj) {
                    if (!(obj instanceof Element))
                        return false;

                    return jaiOps.containsKey(((Element)obj).getName());
                }
            };
        }

        List children = elem.getContent(opFilter);
        if (children.size() != 1) {
            throw new XMLParseException("Expecting exactly one child element",
                    elem);
        }
        Element opElem = (Element)children.get(0);
        String operation = opElem.getName().toLowerCase();
        X_OperationJAI op = (X_OperationJAI)jaiOps.get(operation);
        if (op == null) {
            throw new XMLParseException("Unknown JAI operation: " + operation,
                    elem);
        }

        PendingParameterBlockJAI ppbj = op.parseOp(opElem, altData);

        PendingRenderedOp pj = new PendingRenderedOp(ppbj);

        FImageLayout il = (FImageLayout)
                getHandler(FImageLayout.class).parseChildren(elem,
                new ElementFilter("fImageLayout"), altData, false);

        if (numChFilter == null) {
            numChFilter =
                    getHandler(NumberOfChannels.class).getXMLChildFilter(false);
        }
        NumberOfChannels numChan = (NumberOfChannels)
                getHandler(NumberOfChannels.class).parseChildren(elem,
                        numChFilter, altData, false);
        if (numChan != null) {
            pj.setNumberOfChannels(numChan.numberOfChannels);
        }

        if (il != null) {
            pj.setImageLayout(il);
        }
        else {
            Element propElem = elem.getChild("propagateImageLayout");
            if (propElem != null) {
                int sourceIndex = 0;
                Attribute sourceAttr = propElem.getAttribute("sourceIndex");
                if (sourceAttr != null) {
                    try {
                        sourceIndex = sourceAttr.getIntValue();
                    }
                    catch (DataConversionException e) {
                        throw new XMLParseException("source must be an integer",
                            propElem, e);
                    }
                }
                pj.propagateImageLayout(sourceIndex);
            }
        }


        return pj;
    }

}
