package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;
import org.jdom.Attribute;
import org.jdom.filter.Filter;
import java.util.Random;


/**
 *  XML wrapper for PolynomialNoiseFunction.
 */
public class X_PolynomialNoiseFunction extends XMLHandlerStd {

    static Filter randFilter = null;
    static Filter termsFilter = null;

    public X_PolynomialNoiseFunction()  throws XMLUnhandledTypeException {
        super(PolynomialNoiseFunction.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is a custom JAI operation " +
                "that adds Gaussian noise.";
    }


    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * PolynomialNoiseFunction.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        Element chIndAttr = XMLUtils.xsdAttribute("channelsIndependent",
                "xsd:boolean", false);
        chIndAttr.setAttribute("default", "true");

        Element schema = XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  getHandler(Random.class).schemaInContext("rand", false,
                          false),
                  getHandler(PolynomialNoiseTerm[].class).schemaInContext(
                          "term", true, true),
                  } ));
        schema.addContent(chIndAttr);

        return schema;
    }


    /**
     *  Given a jdom Element object (with children), construct a
     *  PolynomialNoiseFunction instance.
     *
     *  @param elem  jdom element describing the object to be instantiated.
     *  @return  the instantiated PolynomialNoiseFunction object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        boolean channelsIndependent = true;
        Attribute ciAttr = elem.getAttribute("channelsIndependent");
        if (ciAttr != null) {
            try {
                channelsIndependent = ciAttr.getBooleanValue();
            }
            catch (org.jdom.DataConversionException e) {
                throw new XMLParseException("Attribute error", elem, e);
            }
        }


        if (randFilter == null) {
            randFilter = getHandler(Random.class).getXMLChildFilter("rand",
                    false);
        }
        Random rand = (Random)
                getHandler(Random.class).parseChildren(elem, 
                randFilter, altData, false);
        if (rand == null) {
            rand = new Random();
        }

        if (termsFilter == null) {
            termsFilter = getHandler(
                    PolynomialNoiseTerm[].class).getXMLChildFilter("term",
                    false);
        }
        PolynomialNoiseTerm[] terms = (PolynomialNoiseTerm[])
                getHandler(PolynomialNoiseTerm[].class).parseChildren(elem,
                termsFilter, altData, true);

        return new PolynomialNoiseFunction(rand, terms, channelsIndependent);
    }

}
