package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;
import org.jdom.filter.Filter;


/**
 *  XML wrapper for PolynomialTerm.
 */
public class X_PolynomialNoiseTerm extends XMLHandlerStd {

    public X_PolynomialNoiseTerm()  throws XMLUnhandledTypeException {
        super(PolynomialNoiseTerm.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " Describes a term in a noise " +
                "polynomial.";
    }


    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * PolynomialTerm.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdElement("exponent", "xsd:float"),
                  XMLUtils.xsdElement("stddev", "xsd:float")
                  } ));
    }


    /**
     *  Given a jdom Element object (with children), construct a
     *  PolynomialTerm instance.
     *
     *  @param elem  jdom element describing the object to be instantiated.
     *  @return  the instantiated PolynomialTerm object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        float exponent = XMLUtils.getChildFloat(elem, "exponent");
        float stddev = XMLUtils.getChildFloat(elem, "stddev");

        return new PolynomialNoiseTerm(exponent, stddev);
    }

}
