package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.io.File;
import java.io.FileInputStream;
import java.util.Iterator;
import java.util.Properties;

import org.jdom.Element;

/**
 *  XML wrapper for Properties.
 */
public class X_Properties extends XMLHandlerStd {

    public X_Properties()  throws XMLUnhandledTypeException {
        super(Properties.class);
    }


    public String typeDescription() {
        return "Standard java " + this.getClass().getName() + " class.";
    }


    private XMLChildFilter loadFilter = null;


    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * FeaturesSpecs.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        Element propElem = XMLUtils.xsdElement("property", 
                XMLUtils.xsdComplexType(null, 
                  XMLUtils.xsdGrouping("sequence", new Element[] {
                    XMLUtils.xsdElement("key", "xsd:string"),
                    XMLUtils.xsdElement("value", "xsd:string") } ) ) );
        propElem.setAttribute("maxOccurs", "unbounded");

        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("choice", new Element[] {  propElem,
                  getHandler(File.class).schemaInContext(
                    "loadFile", false, true) } ) );
    }


    /**
     * Given a jdom Element object (with children), construct a FeaturesSpecs
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated FeaturesSpecs object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        Properties props = new Properties();

        if (loadFilter == null) {
            loadFilter = getHandler(File.class).getXMLChildFilter("loadFile",
                    false);
        }
        File loadFile = (File) getHandler(File.class).parseChildren(elem,
                loadFilter, altData, false);

        if (loadFile != null) {
            try {
                props.load( new FileInputStream(loadFile) );
            }
            catch (Exception e) {
                throw new XMLParseException("IO error", elem, e);
            }
        }

        Iterator indProps = elem.getChildren("property").iterator();
        while (indProps.hasNext()) {
            Element property = (Element)indProps.next();
            props.setProperty(XMLUtils.getChildString(property, "key"),
                    XMLUtils.getChildString(property, "value"));
        }

        return props;
    }

}
