package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import org.jdom.Element;

/**
 *  XML wrapper for RImageFileLibrary.
 */
public class X_RImageFileLibrary extends XMLHandlerStd {

    public X_RImageFileLibrary() throws XMLUnhandledTypeException {
        super(RImageFileLibrary.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is a library of image files, " +
                "either in a directory or as specified in a hash " +
                "containing a map to the image file locations.";
    }


    private XMLChildFilter propFilter = null;
    private XMLChildFilter dirFilter = null;
    private XMLChildFilter extFilter = null;
    private XMLChildFilter patFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("choice", new Element[] {
                  XMLUtils.xsdGrouping("sequence", new Element[] {
                    getHandler(File.class).schemaInContext("directory", true,
                            true),
                    XMLUtils.xsdGrouping("choice", new Element[] {
                      getHandler(String[].class).schemaInContext("extensions",
                              true, true),
                      getHandler(String.class).schemaInContext("pattern",
                              true, true) } ) } ),
                  getHandler(Properties.class).schemaInContext("mapProps",
                          true, true) } ));
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (propFilter == null) {
            propFilter = getHandler(Properties.class).getXMLChildFilter(
                    "mapProps", true);
        }
        Properties mapProps = (Properties)
                getHandler(Properties.class).parseChildren(elem, propFilter,
                        altData, false);
        if (mapProps != null) {
            HashMap hMap = new HashMap();
            Iterator iter = mapProps.keySet().iterator();
            while (iter.hasNext()) {
                String key = (String)iter.next();
                hMap.put(key, new File((String)mapProps.get(key)));
            }
            return new RImageFileLibrary(hMap);
        }
        else {
            if (dirFilter == null) {
                dirFilter = getHandler(File.class).getXMLChildFilter(
                        "directory", true);
            }

            File directory = (File)getHandler(File.class).parseChildren(elem, 
                    dirFilter, altData, true);

            if (extFilter == null) {
                extFilter = getHandler(String[].class).getXMLChildFilter(
                        "extensions", false);
            }
            String[] extensions = (String[])
                    getHandler(String[].class).parseChildren(elem,
                    extFilter, altData, false);
            if (extensions != null) {
                return new RImageFileLibrary(directory, extensions);
            }

            if (patFilter == null) {
                patFilter = getHandler(String.class).getXMLChildFilter(
                        "pattern", false);
            }
            String patternStr = (String)getHandler(String.class).parseChildren(
                    elem, patFilter, altData, false);
            return new RImageFileLibrary(directory, patternStr);
        }
    }

}
