package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;
import org.jdom.Attribute;
import org.jdom.filter.ElementFilter;
import org.jdom.filter.Filter;

/**
 *  XML wrapper for RawIntensitiesData.
 */
public class X_RawIntensitiesData extends XMLHandlerStd {

    public X_RawIntensitiesData() {
        super(RawIntensitiesData.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is a wrapper for access to " +
                "intensities data.  The data could be passed in or read " +
                "from a data source.";
    }


    Filter ibFilter = null;
    Filter isFilter = null;
    Filter cncFilter = null;

    public Element schemaTypedef() throws XMLUnhandledTypeException {
        Element indexBaseElem = getHandler(int.class).schemaInContext(
                "indexBase", false, true);
        Element intensityScaleElem = getHandler(float.class).schemaInContext(
                "intensityScale", false, true);

        Element schema = XMLUtils.xsdComplexType(xmlTypeName(),
                 XMLUtils.xsdGrouping("sequence", new Element[] {
                   getHandler(NumberOfChannels.class).schemaInContext(
                             false, false),
                   indexBaseElem,
                   intensityScaleElem,
                   XMLUtils.xsdGrouping("choice", new Element[] {
                     getHandler(IntensitiesReader.class).schemaInContext(
                             false, false),
                     getHandler(RawIntensitiesRecord[].class).schemaInContext(
                             false, false) } ) }
                 ));

        schema.addContent(XMLUtils.xsdAttribute("coerceNumChannels",
                "xsd:boolean"));

        return schema;
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        RawIntensitiesData rid = null;

        NumberOfChannels oNumCh = (NumberOfChannels)
                getHandler(NumberOfChannels.class).parseChildren(elem,
                        new ElementFilter("numberOfChannels"),
                        altData, false);
        int numChannels = oNumCh.numberOfChannels;

        boolean coerceNumChannels = false;

        Attribute cncAttr = elem.getAttribute("coerceNumChannels");
        if (cncAttr != null) {
            try {
                coerceNumChannels = cncAttr.getBooleanValue();
            }
            catch (org.jdom.DataConversionException e) {
                throw new XMLParseException(
                        "coerceNumChannels: boolean syntax error", elem, e);
            }
        }
         

        if (ibFilter == null) {
            ibFilter = getHandler(Integer.class).getXMLChildFilter("indexBase",
                   true);
        }

        Integer indexBaseInt = (Integer)
                getHandler(Integer.class).parseChildren(elem, ibFilter, 
                        altData, false);

        int indexBase = (indexBaseInt != null) ? indexBaseInt.intValue() : 0;

        if (isFilter == null) {
            isFilter = getHandler(Float.class).getXMLChildFilter(
                    "intensityScale", true);
        }
        Float intensityScaleFl = (Float)
                getHandler(Float.class).parseChildren(elem, isFilter, altData,
                        false);
        float intensityScale = (intensityScaleFl != null) ?
                intensityScaleFl.floatValue() : 1f;
  
        IntensitiesReader ir = (IntensitiesReader)
                getHandler(IntensitiesReader.class).parseChildren(elem,
                        new ElementFilter("intensitiesReader"),
                        altData, false);

        if (ir != null) {
            try {
                rid = new RawIntensitiesData(ir, numChannels, indexBase,
                        intensityScale, coerceNumChannels);
            }
            catch (DatException e) {
                throw new XMLParseException("Raw data problem", null, e);
            }
        }

        else {
            RawIntensitiesRecord[] recs = (RawIntensitiesRecord[])
                    getHandler(RawIntensitiesRecord[].class).parseChildren(
                            elem,
                            new ElementFilter("rawIntensitiesRecord"),
                            altData, false);
            try {
                rid = new RawIntensitiesData(recs, numChannels, indexBase,
                        intensityScale, coerceNumChannels);
            }
            catch (DatException e) {
                throw new XMLParseException("Raw data problem", null, e);
            }
        }
        
        return rid;
    }

}
