package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Attribute;
import org.jdom.DataConversionException;
import org.jdom.Element;
import org.jdom.filter.ElementFilter;

/**
 *  XML wrapper for RawIntensitiesRecord.
 */
public class X_RawIntensitiesRecord extends XMLHandlerStd {

    public X_RawIntensitiesRecord() {
        super(RawIntensitiesRecord.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " encapsulates the raw " +
                "intensities data for one feature as well as its location " +
                "(indices) in the microarray grid.  These objects are " +
                "sortable by position on the microarray.";
    }

    
    /**
     * Produces an 'xsd:element' component of an XML schema for the
     * FeaturesSpecs.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {

        Element coordsComp = XMLUtils.xsd("complexType");
        coordsComp.addContent(XMLUtils.xsdAttribute("zoneIndex", "xsd:int"));
        coordsComp.addContent(XMLUtils.xsdAttribute("metarow", "xsd:int"));
        coordsComp.addContent(XMLUtils.xsdAttribute("metacol", "xsd:int"));
        coordsComp.addContent(XMLUtils.xsdAttribute("row", "xsd:int", true));
        coordsComp.addContent(XMLUtils.xsdAttribute("col", "xsd:int", true));

        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdElement("coordinates", coordsComp),
                  getHandler(float[].class).schemaInContext("intensities",
                          true, false) 
                  } ) );
    }


    /**
     * Given a jdom Element object (with children), construct a FeaturesSpecs
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated FeaturesSpecs object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        RawIntensitiesRecord iRec = new RawIntensitiesRecord();

        Element coordsElem = elem.getChild("coordinates");
        if (coordsElem == null) {
            throw new XMLParseException("***", elem);
        }

        try {
            Attribute ziAttr = coordsElem.getAttribute("zoneIndex");
            iRec.zoneIndex = (ziAttr != null) ? ziAttr.getIntValue() : -1;

            Attribute mrAttr = coordsElem.getAttribute("metarow");
            iRec.metarow = (mrAttr != null) ? mrAttr.getIntValue() : -1;

            Attribute mcAttr = coordsElem.getAttribute("metacol");
            iRec.metacol = (mcAttr != null) ? mcAttr.getIntValue() : -1;

            Attribute rAttr = coordsElem.getAttribute("row");
            if (rAttr == null) {
                throw new XMLParseException("***", coordsElem);
            }
            iRec.row = rAttr.getIntValue();

            Attribute cAttr = coordsElem.getAttribute("col");
            if (cAttr == null) {
                throw new XMLParseException("***", coordsElem);
            }
            iRec.col = cAttr.getIntValue();
        }
        catch (DataConversionException e) {
            throw new XMLParseException("***", elem, e);
        }

        iRec.intensities = (float[])
                getHandler(float[].class).parseChildren(elem,
                    new ElementFilter("intensities"), altData, true);

        return iRec;
    }

}
