package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.image.RenderedImage;
import java.awt.image.renderable.ParameterBlock;
import java.io.File;
import com.sun.media.jai.codec.TIFFDecodeParam;
import javax.media.jai.JAI;
import java.awt.RenderingHints;
import org.jdom.Element;

/**
 *  XML wrapper for RenderedImage.
 *
 * You can get rendered images (that don't conform to the float model)
 * into the system in 3 ways:
 *  1) From an image library.
 *  2) From a specific file.
 *  3) Passed in as an external object.
 */
public class X_RenderedImage extends XMLHandlerStd {

    public X_RenderedImage()  throws XMLUnhandledTypeException {
        super(RenderedImage.class, true);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is a wrapper for the java " +
                "standard java.awt.image.RenderedImage.";
    }


    private XMLChildFilter libFilter = null;
    private XMLChildFilter tagFilter = null;
    private XMLChildFilter fileFilter = null;
    private XMLChildFilter multFilter = null;

    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * RenderedImage.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {

        Element fromLibraryElem = XMLUtils.xsdElement("fromLibrary",
                XMLUtils.xsd("complexType", 
                  XMLUtils.xsdGrouping("sequence", new Element[] {
                    getHandler(RImageLibrary.class).schemaInContext(
                          "imageLibrary", true, false),
                    getHandler(String.class).schemaInContext(
                          "imageTag", true, true) } ) ) );

        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdGrouping("choice", new Element[] {
                    fromLibraryElem,
                    getHandler(File.class).schemaInContext (
                            "imageFile", true, false) } ),
                  getHandler(float.class).schemaInContext(
                    "multiplyBy", false, true) })
                );
    }


    /**
     * Given a jdom Element object (with children), construct a RenderedImage
     * instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated RenderedImage object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        Element fromLibraryElem = elem.getChild("fromLibrary");
        RenderedImage rim = null;

        if (fromLibraryElem != null) {
            if (libFilter == null) {
                libFilter = getHandler(RImageLibrary.class).getXMLChildFilter(
                        "imageLibrary", true);
            }
            RImageLibrary library = (RImageLibrary)
                    getHandler(RImageLibrary.class).parseChildren(
                            fromLibraryElem, libFilter, altData, true);

            if (tagFilter == null) {
                tagFilter = getHandler(String.class).getXMLChildFilter(
                        "imageTag", true);
            }
            String imageTag = (String)
                    getHandler(String.class).parseChildren(fromLibraryElem,
                        tagFilter, altData, true);
            rim = library.getImage(imageTag);
        }
   
        else {
            if (fileFilter == null) {
                fileFilter = getHandler(File.class).getXMLChildFilter(
                            "imageFile", false);
            }

            File imageFile = (File)
                    getHandler(File.class).parseChildren(elem, fileFilter,
                            altData, true);
            String fName = imageFile.getPath();
            try {
                ParameterBlock pb = new ParameterBlock();
                pb.add(fName);
                pb.add(new TIFFDecodeParam());
                rim = JAI.create("FileLoad", pb);
            }
            catch (Exception e) {
                throw new XMLParseException("Failure in reading file " + fName,
                        elem, e);
            }
        }

        if (multFilter == null) {
            multFilter = getHandler(Float.class).getXMLChildFilter(
                    "multiplyBy", true);
        }

        Float multiplyBy = (Float)
                getHandler(Float.class).parseChildren(elem, multFilter,
                        altData, false);

        if (multiplyBy != null) {

            RenderingHints rHints = FloatSampleModel.getRenderingHints(
                    rim.getMinX(), rim.getMinY(), 
                    rim.getWidth(), rim.getHeight(), 1);

            ParameterBlock pb = new ParameterBlock();
            pb.add( new double[] { multiplyBy.doubleValue() });
            pb.addSource(rim);
            rim = JAI.create("multiplyConst", pb, rHints);
        }

        return rim;
    }

}
