package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for Scan.
 */
public class X_Scan extends XMLHandlerStd {

    public X_Scan() throws XMLUnhandledTypeException {
        super(Scan.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is the top class in the " +
                "hierarchy of data describing a simulated microarray scan.";
    }


    private XMLChildFilter scannerFilter = null;
    private XMLChildFilter microarrayFilter = null;
    private XMLChildFilter fileStemFilter = null;
    private XMLChildFilter gainFilter = null;


    public Element schemaTypedef()
            throws XMLUnhandledTypeException {

        Element gainElem = XMLUtils.xsdElement("gain", "xsd:float");
        gainElem.setAttribute("minOccurs", "0");
        gainElem.setAttribute("default", "1");

        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  getHandler(PendingRenderedOp.class).schemaInContext(
                          "microarray", true, false),
                  getHandler(Scanner.class).schemaInContext(true, false),
                  getHandler(Float.class).schemaInContext("gain", false, true),
                  getHandler(String.class).schemaInContext("fileStem",
                          true, true) } ) );
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (fileStemFilter == null) {
            fileStemFilter = getHandler(String.class).getXMLChildFilter( 
                    "fileStem", true);
        }
        String fileStem = (String)
                XMLHandler.getHandler(String.class).parseChildren(elem, 
                fileStemFilter, altData, true);

        if (scannerFilter == null) {
            scannerFilter = getHandler(Scanner.class).getXMLChildFilter(false);
        }
        Scanner scanner = (Scanner)
                XMLHandler.getHandler(Scanner.class).parseChildren(elem,
                scannerFilter, altData, true);

        if (gainFilter == null) {
            gainFilter =
                    getHandler(Float.class).getXMLChildFilter("gain", true);
        }
        Float gainFl = (Float)
                XMLHandler.getHandler(Float.class).parseChildren(elem,
                gainFilter, altData, false);
        float gain = (gainFl != null) ? gainFl.floatValue() : 1f;

        if (microarrayFilter == null) {
            microarrayFilter =
                    getHandler(PendingRenderedOp.class).getXMLChildFilter(
                            "microarray", false);
        }
        PendingRenderedOp microarray = (PendingRenderedOp)
                XMLHandler.getHandler(PendingRenderedOp.class).parseChildren(
                        elem, microarrayFilter, altData, true);

        return new Scan(microarray, scanner, fileStem, gain);
    }

}
