package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for Scanner.
 */
public class X_Scanner extends XMLHandlerStd {

    public X_Scanner() throws XMLUnhandledTypeException {
        super(Scanner.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is an image scanner class, " +
                "used by Scan class (which calls saveImages method).";
    }


    private XMLChildFilter psFilter = null;
    private XMLChildFilter ssFilter = null;
    private XMLChildFilter nfFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {
        Element gbComp = XMLUtils.xsdComplexType(null, XMLUtils.xsdGrouping(
                "sequence", new Element[] {
                XMLUtils.xsdElement("kernelSize", "xsd:int"),
                XMLUtils.xsdElement("stdDev", "xsd:float") } ) );

        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  getHandler(float.class).schemaInContext("pixelSize",
                          true, true),
                  getHandler(PointModifyFunction.class).schemaInContext(
                          "noiseFunction", false, true),
                  XMLUtils.xsdElement("gBlur", gbComp, false),
                  getHandler(ScanSaveType[].class).schemaInContext(
                          "scanSaveType", true, false)  } ));
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (psFilter == null) {
            psFilter = new XMLChildFilter("pixelSize", true, false);
        }
        Float pixelSize = (Float)getHandler(float.class).parseChildren(elem,
                psFilter, altData, true);

        if (ssFilter == null) {
            ssFilter = getHandler(ScanSaveType[].class).getXMLChildFilter(
                    "scanSaveType", false);
        }
        ScanSaveType[] saveTypes = (ScanSaveType[])
                getHandler(ScanSaveType[].class).parseChildren(elem, 
                        ssFilter, altData, true);
 
        Scanner scanner = new Scanner(saveTypes, pixelSize.floatValue());

        if (nfFilter == null) {
            nfFilter = getHandler(PointModifyFunction.class).getXMLChildFilter(
                    "noiseFunction", true);
        }
        PointModifyFunction noiseFunction = (PointModifyFunction)
                getHandler(PointModifyFunction.class).parseChildren(elem, 
                        nfFilter, altData, false);
        if (noiseFunction != null) {
            scanner.setNoiseFunction(noiseFunction);
        }

        Element gbElem = elem.getChild("gBlur");
        if (gbElem != null) {
            scanner.setBlur(XMLUtils.getChildInteger(gbElem, "kernelSize"),
                    XMLUtils.getChildFloat(gbElem, "stdDev") );
        }
        return scanner;
    }

}
