package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.image.RenderedImage;

import org.jdom.Element;

/**
 *  XML wrapper for SpotterTipConst.
 */
public class X_SpotterTipConst extends XMLHandlerStd {

    public X_SpotterTipConst() throws XMLUnhandledTypeException {
        super(SpotterTipConst.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is an implementation of " +
                "SpotterTip that uses a constant image.";
    }


    private XMLChildFilter locEffFilter = null;
    private XMLChildFilter imageFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  getHandler(RenderedImage.class).schemaInContext("tipTexture",
                          true, false),
                  XMLUtils.xsdElement("physicalTextureSize", "xsd:float",
                          false),
                  getHandler(SpotterTipLocationEffects.class).schemaInContext(
                          false, false)
                  } ));
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (imageFilter == null) {
            imageFilter = getHandler(RenderedImage.class).getXMLChildFilter(
                    "tipTexture", false);
        }
        RenderedImage tipTexture = (RenderedImage)
                getHandler(RenderedImage.class).parseChildren( elem,
                        imageFilter, altData, false);

        float physicalTextureSize =
                XMLUtils.getChildFloat(elem, "physicalTextureSize", 0f);
        SpotterTipConst tip =
                new SpotterTipConst(tipTexture, physicalTextureSize);

        if (locEffFilter == null) {
            locEffFilter = getHandler(
                    SpotterTipLocationEffects.class).getXMLChildFilter(false);
        }
        SpotterTipLocationEffects locEffects = (SpotterTipLocationEffects)
                getHandler(SpotterTipLocationEffects.class).parseChildren(elem,
                  locEffFilter, altData, false);

        if (locEffects != null) {
            tip.setLocationEffects(locEffects);
        }

        return tip;
    }

}
