package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import javax.media.jai.ImageFunction;

import org.jdom.Element;

/**
 *  XML wrapper for SpotterTipFunction.
 */
public class X_SpotterTipFunction extends XMLHandlerStd {

    public X_SpotterTipFunction() throws XMLUnhandledTypeException {
        super(SpotterTipFunction.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is implementation of " + 
                "SpotterTip that uses an ImageFunction.";
    }


    private XMLChildFilter locEffFilter = null;
    private XMLChildFilter funcFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(),
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  getHandler(ImageFunction.class).schemaInContext("tipTexture",
                          true, false),
                  getHandler(SpotterTipLocationEffects.class).schemaInContext(
                          false, false)
                  } ));
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (funcFilter == null) {
            funcFilter = getHandler(ImageFunction.class).getXMLChildFilter(
                    "tipTexture", false);
        }
        ImageFunction imf = (ImageFunction)
                getHandler(ImageFunction.class).parseChildren(elem, funcFilter,
                        altData, false);

        SpotterTipFunction tip = new SpotterTipFunction(imf);

        if (locEffFilter == null) {
            locEffFilter = getHandler(
                    SpotterTipLocationEffects.class).getXMLChildFilter(false);
        }
        SpotterTipLocationEffects locEffects = (SpotterTipLocationEffects)
                getHandler(SpotterTipLocationEffects.class).parseChildren(elem,
                  locEffFilter, altData, false);

        if (locEffects != null) {
            tip.setLocationEffects(locEffects);
        }

        return tip;
    }

}
