package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for SpotterTipLocationEffects.
 */
public class X_SpotterTipLocationEffects extends XMLHandlerStd {

    public X_SpotterTipLocationEffects()  throws XMLUnhandledTypeException {
        super(SpotterTipLocationEffects.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is the encapsulation of " +
                "parameters that effect location of the spotter tip center.";
    }


    /**
     * Produces an 'xsd:element' component of an XML schema for the 
     * SpotterTipLocationEffects.  Note this is not a typedef.
     *
     * @return jdom data structure, rooted with an Element object
     */
    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  XMLUtils.xsdElement("tipWobble", "xsd:float", false),
                  XMLUtils.xsdElement("deltaX", "xsd:float", false),
                  XMLUtils.xsdElement("deltaY", "xsd:float", false) } ) );
    }


    /**
     * Given a jdom Element object (with children), construct a
     * SpotterTipLocationEffects instance.
     *
     * @param elem  jdom element describing the object to be instantiated.
     * @return  the instantiated SpotterTipLocationEffects object
     */
    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        float tipWobble = XMLUtils.getChildFloat(elem, "tipWobble", 0f);
        float deltaX = XMLUtils.getChildFloat(elem, "deltaX", 0f);
        float deltaY = XMLUtils.getChildFloat(elem, "deltaY", 0f);

        return new SpotterTipLocationEffects(deltaX, deltaY, tipWobble);
    }

}
