package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for ZoneCharacteristics.
 */
public class X_ZoneCharacteristics extends XMLHandlerStd {

    public X_ZoneCharacteristics() throws XMLUnhandledTypeException {
        super(ZoneCharacteristics.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " is an encapsulation of " +
                "characteristics of a zone: spotter tip and the " +
                "FeatureCharacteristics for features within this zone.";
    }


    private XMLChildFilter tipFilter = null;
    private XMLChildFilter featureCharFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {
        Element zcElem = XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                   getHandler(SpotterTip.class).schemaInContext(false, false),
                   getHandler(FeatureCharacteristics[].class).schemaInContext(
                           false, false) } ));
        zcElem.addContent(XMLUtils.xsdAttribute("zoneIndex",
                "xsd:nonNegativeInteger", false));
        return zcElem;
    }


    public Object parseElement(Element elem, XMLAltData altData)
            throws XMLParseException, XMLUnhandledTypeException {

        if (tipFilter == null) {
            tipFilter = getHandler(SpotterTip.class).getXMLChildFilter(false);
        }
        SpotterTip tip = (SpotterTip)
                getHandler(SpotterTip.class).parseChildren(elem, tipFilter,
                        altData, false);

        if (featureCharFilter == null) {
            featureCharFilter = getHandler(FeatureCharacteristics.class
                    ).getXMLChildFilter(false);
        }
        FeatureCharacteristics[] featureChars = (FeatureCharacteristics[])
                getHandler(FeatureCharacteristics[].class).parseChildren(elem,
                        featureCharFilter, altData, false);

        ZoneCharacteristics zc = new ZoneCharacteristics(tip, featureChars);

        String zoneIndexStr = elem.getAttributeValue("zoneIndex");
        if (zoneIndexStr != null) {
            try {
                int zoneIndex = Integer.parseInt(zoneIndexStr);
                zc.setZoneIndex(zoneIndex);
            }
            catch (Exception e) {
                throw new XMLParseException(
                        "Integer format error (zoneIndex attribute)", elem, e);
            }
        }

        return zc;
    }

}
