package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import org.jdom.Element;

/**
 *  XML wrapper for ZonesInfoImpl.
 */
public class X_ZonesInfoImpl extends XMLHandlerStd {

    public X_ZonesInfoImpl() throws XMLUnhandledTypeException {
        super(ZonesInfoImpl.class);
    }


    public String typeDescription() {

        String shortName = handledClass.getName();
        int dotPos = shortName.lastIndexOf(".");
        if (dotPos >= 0) {
            shortName = shortName.substring(dotPos+1);
        }
        return shortName + 
         " holds all the " +
                "information specific to a zone, including intensities " +
                "data.";
    }


    private XMLChildFilter fiFilter = null;
    private XMLChildFilter mlFilter = null;
    private XMLChildFilter zcFilter = null;


    public Element schemaTypedef() throws XMLUnhandledTypeException {
        return XMLUtils.xsdComplexType(xmlTypeName(), 
                XMLUtils.xsdGrouping("sequence", new Element[] {
                  getHandler(FeaturesIntensities.class).schemaInContext(
                         true, true),
                  getHandler(MaLayout.class).schemaInContext(true, true),
                  getHandler(ZoneCharacteristics[].class).schemaInContext(
                         false, false) } ) );
    }


    public Object parseElement(Element elem, XMLAltData altData) 
            throws XMLParseException,XMLUnhandledTypeException {

        if (fiFilter == null) {
            fiFilter = getHandler(FeaturesIntensities.class).getXMLChildFilter(
                    false);
        }
        FeaturesIntensities fi = (FeaturesIntensities)
                getHandler(FeaturesIntensities.class).parseChildren(elem,
                        fiFilter, altData, true);

        if (mlFilter == null) {
            mlFilter = getHandler(MaLayout.class).getXMLChildFilter(false);
        }
        MaLayout ml = (MaLayout) getHandler(MaLayout.class).parseChildren(elem,
                        mlFilter, altData, true);
                
        ZonesInfoImpl zi = new ZonesInfoImpl(fi, ml);

        if (zcFilter == null) {
            zcFilter = getHandler(ZoneCharacteristics.class).getXMLChildFilter(
                    false);
        }

        ZoneCharacteristics[] zcs = (ZoneCharacteristics[])
                getHandler(ZoneCharacteristics[].class).parseChildren(elem,
                        zcFilter, altData, false);

        if (zcs != null) {
            if (zcs.length == 1 && zcs[0].zoneIndex == -1) {
                zi.setUniformZoneCharacteristics(zcs[0]);
            }
            else {
                for (int i=0; i<zcs.length; ++i) {
                    zi.setZoneCharacteristics(zcs[i].zoneIndex, zcs[i]);
                }
            }
        }

        return zi;
    }

}
