package org.cybertory.MSS;

/*--------------------------------------------------------
|                   COPYRIGHT (c) 2005
|          Attotron Biotechnologies Corporation
|    Distributed under the GNU general public license:
|     http://www.fsf.org/licensing/licenses/gpl.txt
|
| Avaliable from: http://www.cybertory.org
|         Author: Bill Martin  <eco_bill@yahoo.com>
|      Funded by: NIH SBIR grant #R44 RR13645 02A2
--------------------------------------------------------*/
import java.awt.geom.Point2D;
import java.util.Arrays;

/**
 *  The class holds all the information specific to a zone, including 
 *  intensities data.
 */
public class ZonesInfoImpl implements ZonesInfo {

    private FeaturesIntensities fIntensities;
    private MaLayout mLayout;
    private ZoneCharacteristics[] zonesCharacteristics;

    public ZonesInfoImpl(FeaturesIntensities fIntensities, MaLayout mLayout) {
        this.fIntensities = fIntensities;
        this.mLayout = mLayout;
    }


    public Object getScaled(float pixelSize) {
        ZonesInfoImpl scaled = new ZonesInfoImpl(fIntensities,
                (MaLayout)mLayout.getScaled(pixelSize));
        if (zonesCharacteristics != null) {
            ZoneCharacteristics[] scalesZC =
                    new ZoneCharacteristics[zonesCharacteristics.length];
            for (int i=0; i<scalesZC.length; ++i) {
                if (zonesCharacteristics[i] != null) {
                    scalesZC[i] = (ZoneCharacteristics)
                            zonesCharacteristics[i].getScaled(pixelSize); 
                }
            }
            scaled.setZonesCharacteristics(scalesZC);
        }
        return scaled;
    }


    public void setZonesCharacteristics(
            ZoneCharacteristics[] zonesCharacteristics) {
        this.zonesCharacteristics = zonesCharacteristics;
    }


    public void setZoneCharacteristics(int zoneIndex,
            ZoneCharacteristics zoneCharacteristics) {

        if (this.zonesCharacteristics == null) {
            this.zonesCharacteristics =
                    new ZoneCharacteristics[fIntensities.getNumZones()];
        }
        zonesCharacteristics[zoneIndex] = zoneCharacteristics;
    }


    public void setUniformZoneCharacteristics(
            ZoneCharacteristics zoneCharacteristics) {
        if (this.zonesCharacteristics == null) {
            this.zonesCharacteristics =
                    new ZoneCharacteristics[fIntensities.getNumZones()];
        }
        Arrays.fill(this.zonesCharacteristics, zoneCharacteristics);
    }


    public int getNumZones() {
        return fIntensities.getNumZones();
    }


    public Point2D getZoneLocation(int zoneIndex) {
        return mLayout.getZoneLocation(zoneIndex);
    }


    public float[][][] getZoneIntensities(int zoneIndex) {
        return fIntensities.intensitiesForZone(zoneIndex);
    }


    public ZoneCharacteristics getZoneCharacteristics(int zoneIndex) {
        return (zonesCharacteristics == null) ? null : 
                zonesCharacteristics[zoneIndex];
    }

}
